@@ -1,6 +1,7 @@
 
 do 'web-lib.pl';
 &init_config();
+do 'ui-lib.pl';
 
 # acl_security_form(&options)
 # Output HTML for editing global security options
@@ -88,6 +89,9 @@ if ($o->{'rpc'} == 2) {
 	}
 printf "<input type=radio name=rpc value=0 %s> %s</td> </tr>\n",
 	$o->{'rpc'} == 0 ? "checked" : "", $text{'acl_rpc0'};
+
+print "<tr> <td><b>$text{'acl_readonly'}</b></td>\n";
+print "<td>",&ui_yesno_radio("readonly", $o->{'readonly'}),"</td> </tr>\n";
 }
 
 # acl_security_save(&options)
@@ -110,5 +114,6 @@ $_[0]->{'gedit'} = $in{'gedit_mode'} == 2 ? $in{'gedit_can'} :
 $_[0]->{'gedit2'} = $in{'gedit_mode'} == 4 ? $in{'gedit_gid2'} : undef;
 $_[0]->{'feedback'} = $in{'feedback'};
 $_[0]->{'rpc'} = $in{'rpc'};
+$_[0]->{'readonly'} = $in{'readonly'};
 }
 
