@@ -60,9 +60,7 @@ mkdir($backups_dir, 0700);
 # delete_backup(&backup)
 sub delete_backup
 {
-&lock_file("$backups_dir/$_[0]->{'id'}.backup");
-unlink("$backups_dir/$_[0]->{'id'}.backup");
-&unlock_file("$backups_dir/$_[0]->{'id'}.backup");
+&unlink_logged("$backups_dir/$_[0]->{'id'}.backup");
 }
 
 # parse_backup_url(string)
@@ -187,7 +185,8 @@ elsif ($mode == 4) {
 	}
 }
 
-# execute_backup(&modules, dest, &size, &files)
+# execute_backup(&modules, dest, &size, &files, include-webmin, exclude-files,
+#		 &others)
 # Backs up the configuration files for the modules to the selected destination.
 # The backup is simply a tar file of config files. Returns undef on success,
 # or an error message on failure
@@ -203,35 +202,66 @@ else {
 	$file = &transname();
 	}
 
-# Build list of all files to save, and create per-source-mode manifests
+# Get module descriptions
 local $m;
-local @files;
 local %desc;
-system("rm -rf ".quotemeta($manifests_dir));
-mkdir($manifests_dir, 0755);
-local @manifests;
 foreach $m (@{$_[0]}) {
-	&foreign_require($m, "backup_config.pl");
-	local @mfiles = &foreign_call($m, "backup_config_files");
 	local %minfo = &get_module_info($m);
 	$desc{$m} = $minfo{'desc'};
-	push(@files, @mfiles);
+	}
+
+local @files;
+if (!$_[5]) {
+	# Build list of all files to save from modules
+	foreach $m (@{$_[0]}) {
+		&foreign_require($m, "backup_config.pl");
+		local @mfiles = &foreign_call($m, "backup_config_files");
+		push(@files, @mfiles);
+		push(@{$manifestfiles{$m}}, @mfiles);
+		}
+	}
+
+# Add module config files
+if ($_[4]) {
+	foreach $m (@{$_[0]}) {
+		local @cfiles = ( "$config_directory/$m/config" );
+		push(@files, @cfiles);
+		push(@{$manifestfiles{$m}}, @cfiles);
+		}
+	}
+
+# Add other files
+foreach my $f (@{$_[6]}) {
+	push(@files, $f);
+	push(@{$manifestfiles{"other"}}, $f);
+	}
+
+# Save the manifest files
+&execute_command("rm -rf ".quotemeta($manifests_dir));
+mkdir($manifests_dir, 0755);
+local @manifests;
+foreach $m (@{$_[0]}, "_others") {
+	next if (!defined($manifestfiles{$m}));
 	local $man = "$manifests_dir/$m";
 	&open_tempfile(MAN, ">$man");
-	&print_tempfile(MAN, map { "$_\n" } @mfiles);
+	&print_tempfile(MAN, map { "$_\n" } @{$manifestfiles{$m}});
 	&close_tempfile(MAN);
 	push(@manifests, $man);
 	}
+
+# Make sure we have something to do
 @files = grep { -e $_ } @files;
 @files || (return $text{'backup_enone'});
 
-# Call all module pre functions
-local $m;
-foreach $m (@{$_[0]}) {
-	if (&foreign_defined($m, "pre_backup")) {
-		local $err = &foreign_call($m, "pre_backup", \@files);
-		if ($err) {
-			return &text('backup_epre', $desc{$m}, $err);
+if (!$_[5]) {
+	# Call all module pre functions
+	local $m;
+	foreach $m (@{$_[0]}) {
+		if (&foreign_defined($m, "pre_backup")) {
+			local $err = &foreign_call($m, "pre_backup", \@files);
+			if ($err) {
+				return &text('backup_epre', $desc{$m}, $err);
+				}
 			}
 		}
 	}
@@ -242,23 +272,26 @@ local $qfiles = join(" ", map { s/^\///; quotemeta($_) }
 local $qfile = quotemeta($file);
 local $out;
 if (&has_command("gzip")) {
-	$out = `(cd / ; tar cf - $qfiles | gzip -c >$qfile) 2>&1`;
+	&execute_command("cd / ; tar cf - $qfiles | gzip -c >$qfile",
+			 undef, \$out, \$out);
 	}
 else {
-	$out = `(cd / ; tar cf $qfile $qfiles) 2>&1`;
+	&execute_command("cd / ; tar cf $qfile $qfiles", undef, \$out, \$out);
 	}
 if ($?) {
-	unlink($file) if ($mode != 0);
+	&unlink_file($file) if ($mode != 0);
 	return &text('backup_etar', "<pre>$out</pre>");
 	}
 local @st = stat($file);
 ${$_[2]} = $st[7] if ($_[2]);
 @{$_[3]} = &unique(@files) if ($_[3]);
 
-# Call all module post functions
-foreach $m (@{$_[0]}) {
-	if (&foreign_defined($m, "post_backup")) {
-		&foreign_call($m, "post_backup", \@files);
+if (!$_[5]) {
+	# Call all module post functions
+	foreach $m (@{$_[0]}) {
+		if (&foreign_defined($m, "post_backup")) {
+			&foreign_call($m, "post_backup", \@files);
+			}
 		}
 	}
 
@@ -266,14 +299,14 @@ if ($mode == 1) {
 	# FTP upload to destination
 	local $err;
 	&ftp_upload($host, &date_subs($path), $file, \$err, undef, $user,$pass);
-	unlink($file);
+	&unlink_file($file);
 	return $err if ($err);
 	}
 elsif ($mode == 2) {
 	# SCP to destination
 	local $err;
 	&scp_copy($file, "$user\@$host:".&date_subs($path), $pass, \$err);
-	unlink($file);
+	&unlink_file($file);
 	return $err if ($err);
 	}
 
@@ -298,7 +331,7 @@ else {
 		local $err;
 		&scp_copy("$user\@$host:$path", $file, $pass, \$err);
 		if ($err) {
-			unlink($file);
+			&unlink_file($file);
 			return $err;
 			}
 		}
@@ -307,7 +340,7 @@ else {
 		local $err;
 		&ftp_download($host, $path, $file, \$err, undef, $user, $pass);
 		if ($err) {
-			unlink($file);
+			&unlink_file($file);
 			return $err;
 			}
 		}
@@ -328,9 +361,10 @@ if ($gzipped) {
 else {
 	$cmd = "tar tf $qfile";
 	}
-local $out = `$cmd 2>&1`;
+local $out;
+&execute_command($cmd, undef, \$out, \$out, 0, 1);
 if ($?) {
-	unlink($file) if ($mode != 0);
+	&unlink_file($file) if ($mode != 0);
 	return &text('backup_euntar', "<pre>$out</pre>");
 	}
 local @tarfiles = map { "/$_" } split(/\r?\n/, $out);
@@ -338,14 +372,15 @@ local %tarfiles = map { $_, 1 } @tarfiles;
 
 # Extract manifests for each module
 local %hasmod = map { $_, 1 } @{$_[0]};
-system("rm -rf ".quotemeta($manifests_dir));
+$hasmod{"_others"} = 1;
+&execute_command("rm -rf ".quotemeta($manifests_dir));
 local $rel_manifests_dir = $manifests_dir;
 $rel_manifests_dir =~ s/^\///;
 if ($gzipped) {
-	$out = `(cd / ; gunzip -c $qfile | tar xf - $rel_manifests_dir) 2>&1`;
+	&execute_command("cd / ; gunzip -c $qfile | tar xf - $rel_manifests_dir", undef, \$out, \$out);
 	}
 else {
-	$out = `(cd / ; tar xf $qfile $rel_manifests_dir) 2>&1`;
+	&execute_command("cd / ; tar xf $qfile $rel_manifests_dir", undef, \$out, \$out);
 	}
 opendir(DIR, $manifests_dir);
 local $m;
@@ -367,7 +402,7 @@ while($m = readdir(DIR)) {
 	}
 closedir(DIR);
 if (!@files) {
-	unlink($file) if ($mode != 0);
+	&unlink_file($file) if ($mode != 0);
 	return $text{'backup_enone2'};
 	}
 
@@ -385,7 +420,7 @@ foreach $m (@{$_[0]}) {
 	if (&foreign_defined($m, "pre_restore")) {
 		local $err = &foreign_call($m, "pre_restore", \@files);
 		if ($err) {
-			unlink($file) if ($mode != 0);
+			&unlink_file($file) if ($mode != 0);
 			return &text('backup_epre2', $desc{$m}, $err);
 			}
 		}
@@ -397,14 +432,15 @@ foreach $f (@files) {
 	&lock_file($f);
 	}
 
-# Extract contents (only files specified by modules)
+# Extract contents (only files specified by manifests)
 local $qfiles = join(" ", map { s/^\///; quotemeta($_) }
 				&unique(@files));
 if ($gzipped) {
-	$out = `(cd / ; gunzip -c $qfile | tar xf - $qfiles) 2>&1`;
+	&execute_command("cd / ; gunzip -c $qfile | tar xf - $qfiles",
+			 undef, \$out, \$out);
 	}
 else {
-	$out = `(cd / ; tar xf $qfile $qfiles) 2>&1`;
+	&execute_command("cd / ; tar xf $qfile $qfiles", undef, \$out, \$out);
 	}
 local $ex = $?;
 
@@ -416,7 +452,7 @@ foreach $f (@files) {
 
 # Check for tar error
 if ($ex) {
-	unlink($file) if ($mode != 0);
+	&unlink_file($file) if ($mode != 0);
 	return &text('backup_euntar', "<pre>$out</pre>");
 	}
 
@@ -506,5 +542,30 @@ else {
         }
 }
 
+# show_backup_what(name, webmin?, nofiles?, others)
+# Returns HTML for selecting what gets included in a backup
+sub show_backup_what
+{
+local ($name, $webmin, $nofiles, $others) = @_;
+return &ui_checkbox($name."_webmin", 1, $text{'edit_webmin'}, $webmin)."\n".
+       &ui_checkbox($name."_nofiles", 1, $text{'edit_nofiles'}, !$nofiles)."\n".
+       &ui_checkbox($name."_other", 1, $text{'edit_other'}, $others)."<br>".
+       &ui_textarea($name."_files", join("\n", split(/\t+/, $others)), 3, 50);
+}
+
+# parse_backup_what(name, &in)
+# Returns the webmin and nofiles flags
+sub parse_backup_what
+{
+local ($name, $in) = @_;
+local $webmin = $in->{$name."_webmin"};
+local $nofiles = !$in->{$name."_nofiles"};
+$in->{$name."_files"} =~ s/\r//g;
+local $others = $in->{$name."_other"} ?
+	join("\t", split(/\n+/, $in->{$name."_files"})) : undef;
+$webmin || !$nofiles || $others || &error($text{'save_ewebmin'});
+return ($webmin, $nofiles, $others);
+}
+
 1;
 
