@@ -11,7 +11,9 @@ $backup || die "Failed to find backup $ARGV[0]";
 
 # Do it
 @mods = split(/\s+/, $backup->{'mods'});
-$err = &execute_backup(\@mods, $backup->{'dest'}, \$size);
+$err = &execute_backup(\@mods, $backup->{'dest'}, \$size, undef,
+		       $backup->{'configfile'}, $backup->{'nofiles'},
+		       [ split(/\t+/, $backup->{'others'}) ]);
 
 # Send off the results
 if (($err || $backup->{'emode'} == 0) && $backup->{'email'}) {
