@@ -1,5 +1,6 @@
 
 require 'cron-lib.pl';
+do '../ui-lib.pl';
 
 # acl_security_form(&options)
 # Output HTML for editing security options for the cron module
@@ -37,11 +38,31 @@ printf "<input name=uidmin size=6 value='%s'> -\n",
 printf "<input name=uidmax size=6 value='%s'></td> </tr>\n",
 	$_[0]->{'mode'} == 4 ? $_[0]->{'uidmax'} : "";
 
-print "<tr> <td><b>$text{'acl_control'}</b></td> <td>\n";
-printf "<input type=radio name=allow value=1 %s> $text{'yes'}\n",
-	$_[0]->{'allow'} ? "checked" : "";
-printf "<input type=radio name=allow value=0 %s> $text{'no'}</td> </tr>\n",
-	$_[0]->{'allow'} ? "" : "checked";
+print "<tr> <td><b>$text{'acl_control'}</b></td>\n";
+print "<td>",&ui_radio("allow", $_[0]->{'allow'},
+	[ [ 1, $text{'yes'} ], [ 0, $text{'no'} ] ]),"</td>\n";
+
+print "<td><b>$text{'acl_command'}</b></td>\n";
+print "<td>",&ui_radio("command", $_[0]->{'command'},
+	[ [ 1, $text{'yes'} ], [ 0, $text{'no'} ] ]),"</td> </tr>\n";
+
+print "<tr> <td><b>$text{'acl_create'}</b></td>\n";
+print "<td>",&ui_radio("create", $_[0]->{'create'},
+	[ [ 1, $text{'yes'} ], [ 0, $text{'no'} ] ]),"</td>\n";
+
+print "<td><b>$text{'acl_delete'}</b></td>\n";
+print "<td>",&ui_radio("delete", $_[0]->{'delete'},
+	[ [ 1, $text{'yes'} ], [ 0, $text{'no'} ] ]),"</td> </tr>\n";
+
+print "<tr> <td><b>$text{'acl_move'}</b></td>\n";
+print "<td>",&ui_radio("move", $_[0]->{'move'},
+	[ [ 1, $text{'yes'} ], [ 0, $text{'no'} ] ]),"</td>\n";
+
+print "<td><b>$text{'acl_kill'}</b></td>\n";
+print "<td>",&ui_radio("kill", $_[0]->{'kill'},
+	[ [ 1, $text{'yes'} ], [ 0, $text{'no'} ] ]),"</td> </tr>\n";
+
+print "</tr>\n";
 }
 
 # acl_security_save(&options)
@@ -57,5 +78,10 @@ $_[0]->{'users'} = $in{'mode'} == 0 || $in{'mode'} == 3 ||
 $_[0]->{'uidmin'} = $in{'mode'} == 4 ? $in{'uidmin'} : "";
 $_[0]->{'uidmax'} = $in{'mode'} == 4 ? $in{'uidmax'} : "";
 $_[0]->{'allow'} = $in{'allow'};
+$_[0]->{'command'} = $in{'command'};
+$_[0]->{'create'} = $in{'create'};
+$_[0]->{'delete'} = $in{'delete'};
+$_[0]->{'move'} = $in{'move'};
+$_[0]->{'kill'} = $in{'kill'};
 }
 
