@@ -29,8 +29,8 @@ elsif ($action eq 'delete') {
 		     "<tt>$object</tt>",
 		     "<tt>".&html_escape($p->{'command'})."</tt>");
 	}
-elsif ($action eq 'exec') {
-	return &text($long ? 'log_exec_l' : 'log_exec',
+elsif ($action eq 'exec' || $action eq 'kill') {
+	return &text($long ? 'log_'.$action.'_l' : 'log_'.$action,
 		     "<tt>$object</tt>",
 		     "<tt>".&html_escape($p->{'command'})."</tt>");
 	}
