@@ -19,7 +19,7 @@ $has_e2label = &has_command("e2label");
 $has_xfs_db = &has_command("xfs_db");
 $| = 1;
 
-# list_disks_partitions()
+# list_disks_partitions([include-cds])
 # Returns a structure containing the details of all disks and partitions
 sub list_disks_partitions
 {
@@ -135,7 +135,7 @@ if (open(PARTS, "/proc/partitions")) {
 			if (open(MEDIA, "/proc/ide/hd$n/media")) {
 				local $media = <MEDIA>;
 				close(MEDIA);
-				if ($media =~ /^disk/) {
+				if ($media =~ /^disk/ && !$_[0]) {
 					push(@devs, "/dev/hd$n");
 					}
 				}
@@ -177,7 +177,7 @@ if (open(PARTS, "/proc/partitions")) {
 
 # Call fdisk to get partition and geometry information
 local @cdstat = stat("/dev/cdrom");
-if (@cdstat) {
+if (@cdstat && !$_[0]) {
 	@devs = grep { (stat($_))[1] != $cdstat[1] } @devs;
 	}
 local $devs = join(" ", @devs);
@@ -227,6 +227,7 @@ while(<FDISK>) {
 			$disk->{'lun'} = $5;
 			$disk->{'desc'} = &text('select_scsi',
 						"$2", "$3", "$4", "$5");
+
 			# Work out the SCSI index for this disk
 			local $j;
 			if ($dscsi_mode) {
@@ -366,6 +367,10 @@ foreach $d (@disks) {
 			($d->{'model'} = <MODEL>) =~ s/\r|\n//g;
 			close(MODEL);
 			}
+		if (open(MEDIA, "/proc/ide/$short/media")) {
+			($d->{'media'} = <MEDIA>) =~ s/\r|\n//g;
+			close(MEDIA);
+			}
 		}
 	}
 
@@ -409,6 +414,7 @@ local $rv = &wait_for($fh, 'Partition.*:', 'Selected partition');
 &wait_for($fh, 'Command.*:');
 &wprint("w\n"); sleep(1);
 &close_fdisk();
+undef(@list_disks_partitions_cache);
 }
 
 # delete_partition(disk, partition)
@@ -423,6 +429,7 @@ local $rv = &wait_for($fh, 'Partition.*:', 'Selected partition');
 &wprint("w\n");
 &wait_for($fh, 'Syncing'); sleep(3);
 &close_fdisk();
+undef(@list_disks_partitions_cache);
 }
 
 # create_partition(disk, partition, start, end, type)
@@ -455,6 +462,7 @@ local $rv = &wait_for($fh, 'Partition.*:', 'Selected partition');
 &wprint("w\n");
 &wait_for($fh, 'Syncing'); sleep(3);
 &close_fdisk();
+undef(@list_disks_partitions_cache);
 }
 
 # create_extended(disk, partition, start, end)
@@ -475,6 +483,7 @@ sub create_extended
 &wprint("w\n");
 &wait_for($fh, 'Syncing'); sleep(3);
 &close_fdisk();
+undef(@list_disks_partitions_cache);
 }
 
 # list_tags()
@@ -501,6 +510,7 @@ if ($_[0] eq "4" || $_[0] eq "6" ||
     $_[0] eq "1" || $_[0] eq "e") { @rv = ( "msdos" ); }
 elsif ($_[0] eq "b" || $_[0] eq "c") { @rv = ( "vfat" ); }
 elsif ($_[0] eq "83") { @rv = ( "ext2", "ext3" ); }
+elsif ($_[0] eq "82") { @rv = ( "swap" ); }
 elsif ($_[0] eq "81") { @rv = ( "minix" ); }
 else { return ( ); }
 local %supp = map { $_, 1 } &mount::list_fstypes();
@@ -676,14 +686,14 @@ return $cmd;
 # Returns 1 if this filesystem type can be tuned
 sub can_tune
 {
-return ($_[0] eq "ext2");
+return $_[0] eq "ext2" || $_[0] eq "ext3";
 }
 
 # tunefs_options(type)
 # Output HTML for tuning options for some filesystem type
 sub tunefs_options
 {
-if ($_[0] eq "ext2") {
+if ($_[0] eq "ext2" || $_[0] eq "ext3") {
 	&opt_input("tunefs_c", "", 1);
 
 	print "<td align=right><b>$text{'tunefs_e'}</b></td> <td>\n";
@@ -724,7 +734,7 @@ if ($_[0] eq "ext2") {
 # Returns the tuning command based on user inputs
 sub tunefs_parse
 {
-if ($_[0] eq "ext2") {
+if ($_[0] eq "ext2" || $_[0] eq "ext3") {
 	$cmd = "tune2fs";
 	$cmd .= &opt_check("tunefs_c", '\d+', "-c");
 	$cmd .= $in{'tunefs_e_def'} ? "" : " -e$in{'tunefs_e'}";
@@ -793,6 +803,7 @@ return ();
 sub can_fsck
 {
 return ($_[0] eq "ext2" && &has_command("fsck.ext2") ||
+	$_[0] eq "ext3" && &has_command("fsck.ext3") ||
 	$_[0] eq "minix" && &has_command("fsck.minix"));
 }
 
@@ -800,8 +811,8 @@ return ($_[0] eq "ext2" && &has_command("fsck.ext2") ||
 # Returns the fsck command to unconditionally check a filesystem
 sub fsck_command
 {
-if ($_[0] eq "ext2") {
-	return "fsck -t ext2 -p $_[1]";
+if ($_[0] eq "ext2" || $_[0] eq "ext3") {
+	return "fsck -t $_[0] -p $_[1]";
 	}
 elsif ($_[0] eq "minix") {
 	return "fsck -t minix -a $_[1]";
@@ -1021,7 +1032,7 @@ else { return " $_[2] $in{$_[0]}"; }
 	'f1', 'SpeedStor',
 	'f4', 'SpeedStor large partition',
 	'f2', 'DOS secondary',
-	'fd', 'Linux raid',
+	'fd', 'Linux RAID',
 	'fe', 'LANstep',
 	'ff', 'BBT'
 	);
@@ -1088,7 +1099,7 @@ if ($has_e2label) {
 	}
 if (($? || $label !~ /\S/) && $has_xfs_db) {
 	$label = undef;
-	local $out = `xfs_db -x -p xfs_admin -c label -r $_[0] 2>&1`;
+	local $out = &backquote_with_timeout("xfs_db -x -p xfs_admin -c label -r $_[0] 2>&1", 5);
 	$label = $1 if ($out =~ /label\s*=\s*"(.*)"/ &&
 			$1 ne '(null)');
 	}
