@@ -15,6 +15,20 @@ do '../web-lib.pl';
 &init_config();
 require '../ui-lib.pl';
 
+# Work out init mode
+if ($config{'hostconfig'}) {
+	$init_mode = "osx";
+	}
+elsif ($config{'init_base'}) {
+	$init_mode = "init";
+	}
+elsif ($config{'local_script'}) {
+	$init_mode = "local";
+	}
+elsif ($gconfig{'os_type'} eq 'windows') {
+	$init_mode = "win32";
+	}
+
 # runlevel_actions(level, S|K)
 # Return a list of actions started or stopped in some run-level, each in
 # the format:
@@ -133,10 +147,10 @@ while(-r $file) {
 	}
 &lock_file($file);
 if ($config{soft_links}) {
-	symlink(&action_filename($_[0]), $file);
+	&symlink_file(&action_filename($_[0]), $file);
 	}
 else {
-	link(&action_filename($_[0]), $file);
+	&link_file(&action_filename($_[0]), $file);
 	}
 &unlock_file($file);
 }
@@ -155,9 +169,7 @@ foreach $f (readdir(DIR)) {
 		@stbuf2 = stat("$dir/$f");
 		if ($stbuf[1] == $stbuf2[1]) {
 			# found file to delete.. unlink
-			&lock_file("$dir/$f");
-			unlink("$dir/$f");
-			&unlock_file("$dir/$f");
+			&unlink_logged("$dir/$f");
 			last;
 			}
 		}
@@ -205,8 +217,8 @@ foreach (&action_levels('S', $_[0])) {
 	if (readlink($file)) {
 		# File is a symbolic link.. change it
 		&lock_file($file);
-		unlink($file);
-		symlink("$config{init_dir}/$_[1]", $file);
+		&unlink_file($file);
+		&symlink_file("$config{init_dir}/$_[1]", $file);
 		&unlock_file($file);
 		}
 	if (($idx = index($file, $_[0])) != -1) {
@@ -221,8 +233,8 @@ foreach (&action_levels('K', $_[0])) {
 	if (readlink($file)) {
 		# File is a symbolic link.. change it
 		&lock_file($file);
-		unlink($file);
-		symlink("$config{init_dir}/$_[1]", $file);
+		&unlink_file($file);
+		&symlink_file("$config{init_dir}/$_[1]", $file);
 		&unlock_file($file);
 		}
 	if (($idx = index($file, $_[0])) != -1) {
@@ -352,7 +364,7 @@ return @rv;
 # or 2 if it exists and is enabled
 sub action_status
 {
-if ($config{'init_base'}) {
+if ($init_mode eq "init") {
 	# Look for init script
 	local ($a, $exists, $starting, %daemon);
 	foreach $a (&list_actions()) {
@@ -372,7 +384,7 @@ if ($config{'init_base'}) {
 		}
 	return !$exists ? 0 : $starting ? 2 : 1;
 	}
-else {
+elsif ($init_mode eq "local") {
 	# Look for entry in rc.local
 	local $fn = "$module_config_directory/$_[0].sh";
 	local $cmd = "$fn start";
@@ -384,6 +396,13 @@ else {
 	close(LOCAL);
 	return $found && -r $fn ? 2 : -r $fn ? 1 : 0;
 	}
+elsif ($init_mode eq "win32") {
+	# Look for a win32 service, enabled at boot
+	local @svcs = &list_win32_services();
+	local ($svc) = grep { $_->{'name'} eq $_[0] } @svcs;
+	return !$svc ? 0 :
+	       $svc->{'boot'} == 2 ? 2 : 1;
+	}
 }
 
 # enable_at_boot(action, description, startcode, stopcode, statuscode)
@@ -395,119 +414,120 @@ local $st = &action_status($_[0]);
 return if ($st == 2);	# already starting!
 local ($daemon, %daemon);
 
-if ($config{'daemons_dir'} &&
-    &read_env_file("$config{'daemons_dir'}/$_[0]", \%daemon)) {
-	$daemon++;
-	}
-local $fn;
-if ($config{'init_base'}) {
-	# Normal init.d system
-	$fn = &action_filename($_[0]);
-	}
-else {
-	# Need to create hack init script
-	$fn = "$module_config_directory/$_[0].sh";
-	}
-local @chk = &chkconfig_info($fn);
-local @start = @{$chk[0]} ? @{$chk[0]} : &get_start_runlevels();
-local $start_order = $chk[1] || "9" x $config{'order_digits'};
-local $stop_order = $chk[2] || "9" x $config{'order_digits'};
-local @stop;
-if (@chk) {
-	local %starting = map { $_, 1 } @start;
-	@stop = grep { !$starting{$_} && /^\d+$/ } &list_runlevels();
-	}
-
-local $need_links = 0;
-if ($st == 1 && $daemon) {
-	# Just update daemons file
-	$daemon{'ONBOOT'} = 'yes';
-	&lock_file("$config{'daemons_dir'}/$_[0]");
-	&write_env_file("$config{'daemons_dir'}/$_[0]", \%daemon);
-	&unlock_file("$config{'daemons_dir'}/$_[0]");
-	}
-elsif ($st == 1) {
-	# Just need to create links (later)
-	$need_links++;
-	}
-elsif ($_[1]) {
-	# Need to create the init script
-	&lock_file($fn);
-	&open_tempfile(ACTION, ">$fn");
-	&print_tempfile(ACTION, "#!/bin/sh\n");
-	if ($config{'chkconfig'}) {
-		# Redhat-style description: and chkconfig: lines
-		&print_tempfile(ACTION, "# description: $_[1]\n");
-		&print_tempfile(ACTION, "# chkconfig: $config{'chkconfig'} ",
-			     "$start_order $stop_order\n");
+if ($init_mode eq "init" || $init_mode eq "local") {
+	# In these modes, we create a script to run
+	if ($config{'daemons_dir'} &&
+	    &read_env_file("$config{'daemons_dir'}/$_[0]", \%daemon)) {
+		$daemon++;
 		}
-	elsif ($config{'init_info'}) {
-		# Suse-style init info section
-		&print_tempfile(ACTION, "### BEGIN INIT INFO\n",
-			     "# Provides: $_[0]\n",
-			     "# Required-Start: \$network \$syslog\n",
-			     "# Required-Stop: \$network\n",
-			     "# Default-Start: ",join(" ", @start),"\n",
-			     "# Description: $_[1]\n",
-			     "### END INIT INFO\n");
+	local $fn;
+	if ($init_mode eq "init") {
+		# Normal init.d system
+		$fn = &action_filename($_[0]);
 		}
 	else {
-		&print_tempfile(ACTION, "# $_[1]\n");
+		# Need to create hack init script
+		$fn = "$module_config_directory/$_[0].sh";
+		}
+	local @chk = &chkconfig_info($fn);
+	local @start = @{$chk[0]} ? @{$chk[0]} : &get_start_runlevels();
+	local $start_order = $chk[1] || "9" x $config{'order_digits'};
+	local $stop_order = $chk[2] || "9" x $config{'order_digits'};
+	local @stop;
+	if (@chk) {
+		local %starting = map { $_, 1 } @start;
+		@stop = grep { !$starting{$_} && /^\d+$/ } &list_runlevels();
 		}
-	&print_tempfile(ACTION, "\n");
-	&print_tempfile(ACTION, "case \"\$1\" in\n");
 
-	if ($_[2]) {
-		&print_tempfile(ACTION, "'start')\n");
-		&print_tempfile(ACTION, &tab_indent($_[2]));
-		&print_tempfile(ACTION, "\tRETVAL=\$?\n");
-		if ($config{'subsys'}) {
-			&print_tempfile(ACTION, "\tif [ \"\$RETVAL\" = \"0\" ]; then\n");
-			&print_tempfile(ACTION, "\t\ttouch $config{'subsys'}/$_[0]\n");
-			&print_tempfile(ACTION, "\tfi\n");
+	local $need_links = 0;
+	if ($st == 1 && $daemon) {
+		# Just update daemons file
+		$daemon{'ONBOOT'} = 'yes';
+		&lock_file("$config{'daemons_dir'}/$_[0]");
+		&write_env_file("$config{'daemons_dir'}/$_[0]", \%daemon);
+		&unlock_file("$config{'daemons_dir'}/$_[0]");
+		}
+	elsif ($st == 1) {
+		# Just need to create links (later)
+		$need_links++;
+		}
+	elsif ($_[1]) {
+		# Need to create the init script
+		&lock_file($fn);
+		&open_tempfile(ACTION, ">$fn");
+		&print_tempfile(ACTION, "#!/bin/sh\n");
+		if ($config{'chkconfig'}) {
+			# Redhat-style description: and chkconfig: lines
+			&print_tempfile(ACTION, "# description: $_[1]\n");
+			&print_tempfile(ACTION, "# chkconfig: $config{'chkconfig'} ",
+				     "$start_order $stop_order\n");
 			}
-		&print_tempfile(ACTION, "\t;;\n");
-		}
-
-	if ($_[3]) {
-		&print_tempfile(ACTION, "'stop')\n");
-		&print_tempfile(ACTION, &tab_indent($_[3]));
-		&print_tempfile(ACTION, "\tRETVAL=\$?\n");
-		if ($config{'subsys'}) {
-			&print_tempfile(ACTION, "\tif [ \"\$RETVAL\" = \"0\" ]; then\n");
-			&print_tempfile(ACTION, "\t\trm -f $config{'subsys'}/$_[0]\n");
-			&print_tempfile(ACTION, "\tfi\n");
+		elsif ($config{'init_info'}) {
+			# Suse-style init info section
+			&print_tempfile(ACTION, "### BEGIN INIT INFO\n",
+				     "# Provides: $_[0]\n",
+				     "# Required-Start: \$network \$syslog\n",
+				     "# Required-Stop: \$network\n",
+				     "# Default-Start: ",join(" ", @start),"\n",
+				     "# Description: $_[1]\n",
+				     "### END INIT INFO\n");
 			}
+		else {
+			&print_tempfile(ACTION, "# $_[1]\n");
+			}
+		&print_tempfile(ACTION, "\n");
+		&print_tempfile(ACTION, "case \"\$1\" in\n");
+
+		if ($_[2]) {
+			&print_tempfile(ACTION, "'start')\n");
+			&print_tempfile(ACTION, &tab_indent($_[2]));
+			&print_tempfile(ACTION, "\tRETVAL=\$?\n");
+			if ($config{'subsys'}) {
+				&print_tempfile(ACTION, "\tif [ \"\$RETVAL\" = \"0\" ]; then\n");
+				&print_tempfile(ACTION, "\t\ttouch $config{'subsys'}/$_[0]\n");
+				&print_tempfile(ACTION, "\tfi\n");
+				}
+			&print_tempfile(ACTION, "\t;;\n");
+			}
+
+		if ($_[3]) {
+			&print_tempfile(ACTION, "'stop')\n");
+			&print_tempfile(ACTION, &tab_indent($_[3]));
+			&print_tempfile(ACTION, "\tRETVAL=\$?\n");
+			if ($config{'subsys'}) {
+				&print_tempfile(ACTION, "\tif [ \"\$RETVAL\" = \"0\" ]; then\n");
+				&print_tempfile(ACTION, "\t\trm -f $config{'subsys'}/$_[0]\n");
+				&print_tempfile(ACTION, "\tfi\n");
+				}
+			&print_tempfile(ACTION, "\t;;\n");
+			}
+
+		if ($_[4]) {
+			&print_tempfile(ACTION, "'status')\n");
+			&print_tempfile(ACTION, &tab_indent($_[4]));
+			&print_tempfile(ACTION, "\t;;\n");
+			}
+
+		if ($_[2] && $_[3]) {
+			&print_tempfile(ACTION, "'restart')\n");
+			&print_tempfile(ACTION, "\t\$0 stop && \$0 start\n");
+			&print_tempfile(ACTION, "\tRETVAL=\$?\n");
+			&print_tempfile(ACTION, "\t;;\n");
+			}
+
+		&print_tempfile(ACTION, "*)\n");
+		&print_tempfile(ACTION, "\techo \"Usage: \$0 { start | stop }\"\n");
+		&print_tempfile(ACTION, "\tRETVAL=1\n");
 		&print_tempfile(ACTION, "\t;;\n");
+		&print_tempfile(ACTION, "esac\n");
+		&print_tempfile(ACTION, "exit \$RETVAL\n");
+		&close_tempfile(ACTION);
+		chmod(0755, $fn);
+		&unlock_file($fn);
+		$need_links++;
 		}
 
-	if ($_[4]) {
-		&print_tempfile(ACTION, "'status')\n");
-		&print_tempfile(ACTION, &tab_indent($_[4]));
-		&print_tempfile(ACTION, "\t;;\n");
-		}
-
-	if ($_[2] && $_[3]) {
-		&print_tempfile(ACTION, "'restart')\n");
-		&print_tempfile(ACTION, "\t\$0 stop && \$0 start\n");
-		&print_tempfile(ACTION, "\tRETVAL=\$?\n");
-		&print_tempfile(ACTION, "\t;;\n");
-		}
-
-	&print_tempfile(ACTION, "*)\n");
-	&print_tempfile(ACTION, "\techo \"Usage: \$0 { start | stop }\"\n");
-	&print_tempfile(ACTION, "\tRETVAL=1\n");
-	&print_tempfile(ACTION, "\t;;\n");
-	&print_tempfile(ACTION, "esac\n");
-	&print_tempfile(ACTION, "exit \$RETVAL\n");
-	&close_tempfile(ACTION);
-	chmod(0755, $fn);
-	&unlock_file($fn);
-	$need_links++;
-	}
-
-if ($need_links) {
-	if ($config{'init_base'}) {
+	if ($need_links && $init_mode eq "init") {
 		# Just link up the init script
 		local $s;
 		foreach $s (@start) {
@@ -517,7 +537,7 @@ if ($need_links) {
 			&add_rl_action($_[0], $s, "K", $stop_order);
 			}
 		}
-	else {
+	elsif ($need_links) {
 		# Just add rc.local entry
 		local $lref = &read_file_lines($config{'local_script'});
 		local $i;
@@ -533,6 +553,29 @@ if ($need_links) {
 		&flush_file_lines();
 		}
 	}
+elsif ($init_mode eq "win32") {
+	# Enable and/or create a win32 service
+	if ($st == 1) {
+		# Just enable
+		&enable_win32_service($_[0]);
+		}
+	else {
+		# Need to create service, which calls wrapper program
+		eval "use Win32::Daemon";
+		local $perl_path = &get_perl_path();
+		local %svc = ( 'name' => $_[0],
+			 'display' => $_[1],
+			 'path' => $perl_path,
+			 'user' => '',
+			 'pwd' => $module_root_directory,
+			 'parameters' => "$module_root_directory/win32.pl $_[2]",
+			);
+		if (!Win32::Daemon::CreateService(\%svc)) {
+			print STDERR "Failed to create Win32 service : ",
+			     Win32::FormatMessage(Win32::Daemon::GetLastError()),"\n";
+			}
+		}
+	}
 }
 
 # disable_at_boot(action)
@@ -542,7 +585,7 @@ sub disable_at_boot
 local $st = &action_status($_[0]);
 return if ($st != 2);	# not currently starting
 
-if ($config{'init_base'}) {
+if ($init_mode eq "init") {
 	# Unlink or disable init script
 	local ($daemon, %daemon);
 	local $file = &action_filename($_[0]);
@@ -573,7 +616,7 @@ if ($config{'init_base'}) {
 			}
 		}
 	}
-else {
+elsif ($init_mode eq "local") {
 	# Take out of rc.local file
 	local $lref = &read_file_lines($config{'local_script'});
 	local $cmd = "$module_config_directory/$_[0].sh start";
@@ -597,6 +640,10 @@ else {
 		}
 	&flush_file_lines();
 	}
+elsif ($init_mode eq "win32") {
+	# Disable the service
+	&disable_win32_service($_[0]);
+	}
 }
 
 # tab_indent(lines)
@@ -632,5 +679,104 @@ else {
 	}
 }
 
+# list_win32_services()
+# Returns a list of known Win32 services
+sub list_win32_services
+{
+local @rv;
+local $svc;
+
+# Get the current statuses
+&open_execute_command(SC, "sc query type= service state= all", 1, 1);
+while(<SC>) {
+	s/\r|\n//g;
+	if (/^SERVICE_NAME:\s+(\S+)/) {
+		$svc = { 'name' => $1 };
+		push(@rv, $svc);
+		}
+	elsif (/^DISPLAY_NAME:\s+(\S.*)/ && $svc) {
+		$svc->{'desc'} = $1;
+		}
+	elsif (/^\s+TYPE\s+:\s+(\d+)\s+(\S+)/ && $svc) {
+		$svc->{'type'} = $1;
+		$svc->{'type_desc'} = $2;
+		}
+	elsif (/^\s+STATE\s+:\s+(\d+)\s+(\S+)/ && $svc) {
+		$svc->{'state'} = $1;
+		$svc->{'state_desc'} = $2;
+		}
+	}
+close(SC);
+
+# For each service, see if it starts at boot or not
+foreach $svc (@rv) {
+	&open_execute_command(SC, "sc qc $svc->{'name'}", 1, 1);
+	while(<SC>) {
+		s/\r|\n//g;
+		if (/^\s+START_TYPE\s+:\s+(\d+)\s+(\S+)/) {
+			$svc->{'boot'} = $1;
+			$svc->{'boot_desc'} = $2;
+			}
+		}
+	close(SC);
+	}
+
+return @rv;
+}
+
+# start_win32_service(name)
+# Attempts to start a service, returning undef on success, or some error message
+sub start_win32_service
+{
+local ($name) = @_;
+local $out = &backquote_command("sc start $name 2>&1");
+return $? ? $out : undef;
+}
+
+# stop_win32_service(name)
+# Attempts to stop a service, returning undef on success, or some error message
+sub stop_win32_service
+{
+local ($name) = @_;
+local $out = &backquote_command("sc stop $name 2>&1");
+return $? ? $out : undef;
+}
+
+# enable_win32_service(name)
+# Marks some service as starting at boot time
+sub enable_win32_service
+{
+local ($name) = @_;
+local $out = &backquote_command("sc config $name start= auto 2>&1");
+return $? ? $out : undef;
+}
+
+# disable_win32_service(name)
+# Marks some service as disabled at boot time
+sub disable_win32_service
+{
+local ($name) = @_;
+local $out = &backquote_command("sc config $name start= demand 2>&1");
+return $? ? $out : undef;
+}
+
+# create_win32_service(name, command, desc)
+# Creates a new win32 service, enabled at boot time
+sub create_win32_service
+{
+local ($name, $cmd, $desc) = @_;
+local $out = &backquote_command("sc create $name DisplayName= \"$desc\" type= share start= auto binPath= \"$cmd\" 2>&1");
+return $? ? $out : undef;
+}
+
+# delete_win32_service(name)
+# Delete some existing service
+sub delete_win32_service
+{
+local ($name) = @_;
+local $out = &backquote_command("sc delete $name 2>&1");
+return $? ? $out : undef;
+}
+
 1;
 
