@@ -45,6 +45,9 @@ elsif ($action eq 'massstart' || $action eq 'massstop' ||
        $action eq 'massenable' || $action eq 'massdisable') {
 	return &text('log_'.$action, join(", ", map { "<tt>$_</tt>" } split(/\s+/, $object)));
 	}
+elsif ($action eq 'telinit') {
+	return &text('log_telinit', $object);
+	}
 else {
 	return undef;
 	}
