@@ -199,7 +199,9 @@ while(<CMD>) {
 close(CMD);
 
 # add output from swapinfo
-foreach (split(/\n/, `swapinfo`)) {
+local $out;
+&execute_command("swapinfo", undef, \$out, undef, 0, 1);
+foreach (split(/\n/, $out)) {
 	if (/^(\/\S+)\s+\d+\s+\d+/) {
 		push(@rv, [ "swap", $1, "swap", "-" ]);
 		}
@@ -316,8 +318,13 @@ sub disk_space
 {
 if (&get_mounted($_[1], "*") < 0) { return (); }
 if ($_[0] eq "proc" || $_[0] eq "swap") { return (); }
-`df -k $_[1]` =~ /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/;
-return ($1, $2);
+&execute_command("df -k $_[1]", undef, \$out, undef, 0, 1);
+if ($out =~ /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/) {
+	return ($1, $2);
+	}
+else {
+	return ( );
+	}
 }
 
 
@@ -691,14 +698,14 @@ if ($_[0] eq "nfs") {
 	if ($in{nfs_host} !~ /^\S+$/) {
 		&error("'$in{nfs_host}' is not a valid hostname");
 		}
-	$out = `ping -c 1 '$in{nfs_host}' 2>&1`;
+	&execute_command("ping -c 1 '$in{nfs_host}'", undef, \$out, \$out);
 	if ($out =~ /unknown host/i) {
 		&error("The host '$in{nfs_host}' does not exist");
 		}
 	elsif ($out =~ /100\% packet loss/) {
 		&error("The host '$in{nfs_host}' is down");
 		}
-	$out = `showmount -e '$in{nfs_host}' 2>&1`;
+	&execute_command("showmount -e '$in{nfs_host}'", undef, \$out, \$out);
 	if ($out =~ /Unable to receive/) {
 		&error("The host '$in{nfs_host}' does not support NFS");
 		}
@@ -718,26 +725,24 @@ if ($_[0] eq "nfs") {
 
 	# Try a test mount to see if filesystem is available
 	$temp = &transname();
-	mkdir($temp, 0755);
-	$mout = `mount $in{nfs_host}:$in{nfs_dir} $temp 2>&1`;
+	&make_dir($temp, 0755);
+	&execute_command("mount $in{nfs_host}:$in{nfs_dir} $temp",
+			 undef, \$mout, \$mout);
 	if ($mout =~ /No such file or directory/) {
-		rmdir($temp);
 		&error("The directory '$in{nfs_dir}' does not exist on the ".
 		       "host $in{nfs_host}. The available directories are:".
 		       "<pre>$dirlist</pre>");
 		}
 	elsif ($mout =~ /Permission denied/) {
-		rmdir($temp);
 		&error("This host is not allowed to mount the directory ".
 		       "$in{nfs_dir} from $in{nfs_host}");
 		}
 	elsif ($?) {
-		rmdir($temp);
 		&error("NFS Error - $mout");
 		}
 	# It worked! unmount
-	system("umount $temp");
-	rmdir($temp);
+	&execute_command("umount $temp");
+	&unlink_file($temp);
 	return "$in{nfs_host}:$in{nfs_dir}";
 	}
 elsif ($_[0] eq "smbfs") {
@@ -936,11 +941,11 @@ sub create_swap
 local($out, $bl);
 $bl = $_[1] * ($_[2] eq "g" ? 1024*1024 :
 	       $_[2] eq "m" ? 1024 : 1);
-$out = `dd if=/dev/zero of=$_[0] bs=1024 count=$bl 2>&1`;
+$out = &backquote_logged("dd if=/dev/zero of=$_[0] bs=1024 count=$bl 2>&1");
 if ($?) { return "dd failed : $out"; }
-$out = `mkswap $_[0] $bl 2>&1`;
+$out = &backquote_logged("mkswap $_[0] $bl 2>&1");
 if ($?) { return "mkswap failed : $out"; }
-system("sync >/dev/null 2>&1");
+&system_logged("sync >/dev/null 2>&1");
 return 0;
 }
 
@@ -951,7 +956,7 @@ sub exports_list
 {
 local($dref, $cref, $out, $_);
 $dref = $_[1]; $cref = $_[2];
-$out = `showmount -e $_[0] 2>&1`;
+&execute_command("showmount -e ".quotemeta($_[0]), undef, \$out, \$out, 0, 1);
 if ($?) { return $out; }
 foreach (split(/\n/, $out)) {
 	if (/^(\/\S*)\s+(.*)$/) {
@@ -966,7 +971,7 @@ return undef;
 sub broadcast_addr
 {
 local($out);
-$out = `ifconfig -a 2>&1`;
+&execute_command("ifconfig -a", undef, \$out, \$out, 0, 1);
 if ($out =~ /broadcast\s+(\S+)\s+/) { return $1; }
 return "255.255.255.255";
 }
