@@ -161,15 +161,19 @@ foreach (@autotab) {
 sub list_mounted
 {
 local(@rv, @p, $_, $i, $r);
-foreach (split(/\n/, `swapinfo -a`)) {
+&open_execute_command(SWAP, "swapinfo -a", 1, 1);
+while(<SWAP>) {
 	if (/^dev\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/) { push(@rv, [ "swap", $8, "swap", "pri=$7" ]); }
 	}
-foreach (split(/\n/, `cat /etc/mnttab`)) {
+close(SWAP);
+&open_tempfile(MNTTAB, "/etc/mnttab");
+while(<MNTTAB>) {
 	s/#.*$//g; if (!/\S/) { next; }
 	@p = split(/\s+/, $_);
 	if ($p[2] eq "ignore") { next; }
 	push(@rv, [ $p[1], $p[0], $p[2], $p[3] ]);
 	}
+close(MNTTAB);
 return @rv;
 }
 
@@ -226,8 +230,14 @@ if (&get_mounted($_[1], "*") < 0) { return (); }
 if ($_[0] eq "swap") {
 	return ();
 	}
-`bdf $_[1]` =~ /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/;
-return ($1, $2);
+local $out;
+&execute_command("bdf $_[1]", undef, \$out, \$out, 0, 1);
+if ($out =~ /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/) {
+	return ($1, $2);
+	}
+else {
+	return ( );
+	}
 }
 
 
@@ -676,14 +686,14 @@ if ($_[0] eq "nfs") {
         if ($in{nfs_host} !~ /^\S+$/) {
                 &error("'$in{nfs_host}' is not a valid hostname");
                 }
-        $out = `ping '$in{nfs_host}' -n 1 2>&1`;
+	&execute_command("ping -c 1 '$in{nfs_host}'", undef, \$out, \$out);
         if ($out =~ /unknown host/i) {
                 &error("The host '$in{nfs_host}' does not exist");
                 }
         elsif ($out =~ /100\% packet loss/) {
                 &error("The host '$in{nfs_host}' is down");
                 }
-        $out = `showmount -e '$in{nfs_host}' 2>&1`;
+	&execute_command("showmount -e '$in{nfs_host}'", undef, \$out, \$out);
         if ($out =~ /Unable to receive/) {
                 &error("The host '$in{nfs_host}' does not support NFS");
                 }
@@ -703,26 +713,24 @@ if ($_[0] eq "nfs") {
 
 	# Try a test mount to see if filesystem is available
 	$temp = &transname();
-	mkdir($temp, 0755);
-	$mout = `mount $in{nfs_host}:$in{nfs_dir} $temp 2>&1`;
+	&make_dir($temp, 0755);
+	&execute_command("mount $in{nfs_host}:$in{nfs_dir} $temp",
+			 undef, \$mout, \$mout);
 	if ($mout =~ /No such file or directory/) {
-		rmdir($temp);
 		&error("The directory '$in{nfs_dir}' does not exist on the ".
 		       "host $in{nfs_host}. The available directories are:".
 		       "<pre>$dirlist</pre>");
 		}
 	elsif ($mout =~ /Permission denied/) {
-		rmdir($temp);
 		&error("This host is not allowed to mount the directory ".
 		       "$in{nfs_dir} from $in{nfs_host}");
 		}
 	elsif ($?) {
-		rmdir($temp);
 		&error("NFS Error - $mout");
 		}
 	# It worked! unmount
-	`umount $temp`;
-	rmdir($temp);
+	&execute_command("umount $temp");
+	&unlink_file($temp);
 	return "$in{nfs_host}:$in{nfs_dir}";
 	}
 elsif ($_[0] eq "hfs") {
@@ -898,7 +906,7 @@ local($out, $part, $found);
 # Check if the device/partition actually exists
 if ($_[0] =~ /^\/dev\/dsk\/c(.)t(.)d(.)s(.)$/) {
 	# a normal scsi device..
-	if (!open(DEV, $_[0])) {
+	if (!&open_tempfile(DEV, $_[0], 0, 1)) {
 		if ($! =~ /No such file or directory/) {
                 	&error("The SCSI target for '$_[0]' does not exist");
 			}
@@ -906,18 +914,18 @@ if ($_[0] =~ /^\/dev\/dsk\/c(.)t(.)d(.)s(.)$/) {
                 	&error("The SCSI target for '$_[0]' does not exist");
 			}
 		}
-	close(DEV);
+	&close_tempfile(DEV);
 	}
 elsif ($_[0] =~ /^\/dev\/vg([0-9]+)\/(\S+)$/) {
 	# Logical Volume device..
-	$out = `lvdisplay -v $_[0] 2>&1`;
+	$out = &backquote_command("lvdisplay -v $_[0] 2>&1");
 	if ($out =~ /No such file or directory/) {
 		&error("The Logical Volume device for '$_[0]' does not exist");
 		}
 	}
 else {
 	# Some other device
-	if (!open(DEV, $_[0])) {
+	if (!&open_tempfile(DEV, $_[0], 0, 1)) {
 		if ($! =~ /No such file or directory/) {
 			&error("The device file '$_[0]' does not exist");
 			}
@@ -925,12 +933,12 @@ else {
 			&error("The device for '$_[0]' does not exist");
 			}
 		}
-	close(DEV);
+	&close_tempfile(DEV);
 	}
 
 # Check the filesystem type
 if ($_[1] ne "cdfs" && $_[1] ne "swap") {
-	$out = `fstyp $_[0] 2>&1`;
+	$out = &backquote_command("fstyp $_[0] 2>&1");
 	if ($out =~ /^([A-z0-9]+)\n$/) {
 		if ($1 eq $_[1]) { return; }
 		else {
@@ -1138,9 +1146,9 @@ return @rv ? join(',' , @rv) : "-";
 sub create_swap
 {
 local($out);
-$out = `mkfile $_[1]$_[2] $_[0] 2>&1`;
+$out = &backquote_logged("mkfile $_[1]$_[2] $_[0] 2>&1");
 if ($?) {
-	unlink($_[0]);
+	&unlink_file($_[0]);
 	return "mkfile failed : $out";
 	}
 return 0;
@@ -1154,7 +1162,7 @@ sub exports_list
 {
 local($dref, $cref, $out, $_);
 $dref = $_[1]; $cref = $_[2];
-$out = `showmount -e $_[0] 2>&1`;
+$out = &backquote_command("showmount -e ".quotemeta($_[0])." 2>&1", 1);
 if ($?) { return $out; }
 foreach (split(/\n/, $out)) {
 	if (/^(\/\S*)\s+(.*)$/) {
@@ -1170,7 +1178,7 @@ sub broadcast_addr
 {
 local($out, $hostname, $broadcast, @tmp);
 $hostname = get_system_hostname();
-$out = `netstat -i 2>&1 | grep $hostname`;
+$out = &backquote_command("netstat -i 2>&1 | grep $hostname", 1);
 if ($out =~ /\s+(\S*)\s+(\S*)\s+(.*)/) {
 	$broadcast = "$2.255.255.255";
 	@tmp = split(/\./,$broadcast);
