@@ -194,16 +194,20 @@ if ($found) {
 sub list_mounted
 {
 local(@rv, @p, $_, $i, $r);
-foreach (split(/\n/, `swap -l 2>/dev/null`)) {
+&open_execute_command(SWAP, "swap -l 2>/dev/null", 1, 1);
+while(<SWAP>) {
 	if (/^\s*\d+\s+(\/\S+)/) {
 		push(@rv, [ "swap", $1, "swap", "-" ]);
 		}
 	}
-foreach (split(/\n/, `cat /etc/mtab`)) {
+close(SWAP);
+open(MTAB, "/etc/mtab");
+while(<MTAB>) {
 	s/#.*$//g; if (!/\S/) { next; }
 	@p = split(/\s+/, $_);
 	push(@rv, [ $p[1], $p[0], $p[2], $p[3] ]);
 	}
+close(MTAB);
 return @rv;
 }
 
@@ -263,8 +267,13 @@ if (&get_mounted($_[1], "*") < 0) { return (); }
 if ($_[0] eq "fd" || $_[0] eq "proc" || $_[0] eq "swap" || $_[0] eq "autofs") {
 	return ();
 	}
-`df -k $_[1]` =~ /Mounted on\n\S+\s+\S+\s+(\S+)\s+\S+\s+(\S+)/;
-return ($1, $2);
+if (&backquote_command("df -k $_[1]", 1) =~
+    /Mounted on\n\S+\s+\S+\s+(\S+)\s+\S+\s+(\S+)/) {
+	return ($1, $2);
+	}
+else {
+	return ( );
+	}
 }
 
 
@@ -615,14 +624,14 @@ sub check_location
 {
 if ($_[0] eq "nfs") {
 	# Use ping and showmount to see if the host exists and is up
-	local $out = `ping -c 1 '$in{nfs_host}' 2>&1`;
+	local $out = &backquote_command("ping -c 1 '$in{nfs_host}' 2>&1");
 	if ($out =~ /unknown host/i) {
 		&error(&text('irix_ehost', $in{'nfs_host'}));
 		}
 	elsif ($out =~ /100\% packet loss/) {
 		&error(&text('irix_eloss', $in{'nfs_host'}));
 		}
-	$out = `showmount -e '$in{nfs_host}' 2>&1`;
+	$out = &backquote_command("showmount -e '$in{nfs_host}' 2>&1");
 	if ($out =~ /port mapper failure/i) {
 		&error(&text('irix_eshowmount', $in{'nfs_host'}));
 		}
@@ -641,25 +650,24 @@ if ($_[0] eq "nfs") {
 
 	# Try a test mount to see if filesystem is available
 	local $temp = &transname();
-	mkdir($temp, 0755);
-	local $mout = `mount $in{'nfs_host'}:$in{'nfs_dir'} $temp 2>&1`;
+	&make_dir($temp, 0755);
+	local $mout;
+	&execute_command("mount $in{'nfs_host'}:$in{'nfs_dir'} $temp 2>&1",
+			 undef, \$mout, \$mout);
 	if ($mout =~ /No such file or directory/i) {
-		rmdir($temp);
 		&error(&text('irix_enotexist', $in{'nfs_dir'}, $in{'nfs_host'},
 			     "<pre>$dirlist</pre>"));
 		}
 	elsif ($mout =~ /Permission denied/i) {
-		rmdir($temp);
 		&error(&text('irix_epermission', $in{'nfs_dir'},
 			     $in{'nfs_host'}));
 		}
 	elsif ($?) {
-		rmdir($temp);
 		&error(&text('irix_enfserr', $mout));
 		}
 	# It worked! unmount
-	system("umount $temp");
-	rmdir($temp);
+	&execute_command("umount $temp");
+	&unlink_file($temp);
 	return "$in{nfs_host}:$in{nfs_dir}";
 	}
 elsif ($_[0] eq "autofs") {
@@ -700,7 +708,7 @@ else {
 
 	# Check the filesystem type
 	if ($_[0] eq "xfs" || $_[0] eq "efs") {
-		local $out = `fstyp '$dv' 2>&1`;
+		local $out = &backquote_command("fstyp '$dv' 2>&1");
 		$out =~ s/[^A-Za-z0-9]//g;
 		if ($?) {
 			&error(&text('irix_efstyp2', $dv, $out));
@@ -827,7 +835,7 @@ sub create_swap
 local($out);
 $out = &backquote_logged("mkfile $_[1]$_[2] $_[0] 2>&1");
 if ($?) {
-	unlink($_[0]);
+	&unlink_file($_[0]);
 	return "mkfile failed : $out";
 	}
 return 0;
@@ -841,7 +849,7 @@ sub exports_list
 {
 local($dref, $cref, $out, $_);
 $dref = $_[1]; $cref = $_[2];
-$out = `showmount -e $_[0] 2>&1`;
+$out = &backquote_command("showmount -e ".quotemeta($_[0]), 1);
 if ($?) { return $out; }
 foreach (split(/\n/, $out)) {
 	if (/^(\/\S*)\s+(.*)$/) {
@@ -856,7 +864,7 @@ return undef;
 sub broadcast_addr
 {
 local($out);
-$out = `ifconfig -a 2>&1`;
+$out = &backquote_command("ifconfig -a 2>&1", 1);
 if ($out =~ /broadcast\s+(\S+)/) { return $1; }
 return "255.255.255.255";
 }
