@@ -72,6 +72,7 @@ if ($_[1] ne "-") {
 		else { $options{$_} = ""; }
 		}
 	}
+return \%options;
 }
 
 # join_options(type, [&hash])
@@ -191,22 +192,21 @@ return &indexof($_[0], @types) >= 0;
 sub compile_program
 {
 return if (-r "$module_config_directory/$_[0]" &&
-   system("$module_config_directory/$_[0] >/dev/null 2>&1 </dev/null")
-	== 0);
-local $arch = `uname -m`;
+   &execute_command("$module_config_directory/$_[0]", undef, undef, undef, 0, 1) == 0);
+local $arch = &backquote_command("uname -m");
 $arch =~ s/\r|\n//g;
 local $re = $_[1];
 if ($re && $arch =~ /^$re$/i && -r "$module_root_directory/$_[0]" &&
-system("$module_root_directory/$_[0] >/dev/null 2>&1 </dev/null") == 0) {
-# Compiled program for this architecture already exists and is working,
-# so can just copy
-system("cp $module_root_directory/$_[0] $module_config_directory/$_[0]");
-}
+    &execute_command("$module_root_directory/$_[0]", undef, undef, undef, 0, 1) == 0) {
+	# Compiled program for this architecture already exists and is working,
+	# so can just copy
+	&execute_command("cp $module_root_directory/$_[0] $module_config_directory/$_[0]");
+	}
 else {
 	# Need to compile
 	local ($cc) = (&has_command("gcc") || &has_command("cc"));
 	$cc || &error($text{'egcc'});
-	local $out = `$cc -o $module_config_directory/$_[0] $module_root_directory/$_[0].c 2>&1`;
+	local $out = &backquote_logged("$cc -o $module_config_directory/$_[0] $module_root_directory/$_[0].c 2>&1");
 	if ($?) {
 		&error(&text('ecompile', "<pre>$out</pre>"));
 		}
@@ -254,5 +254,35 @@ if (&can_delete_directory($_[0]) &&
 	}
 }
 
+# remount_dir(directory, device, type, options)
+# Adjusts the options for some mounted filesystem
+sub remount_dir
+{
+if (defined(&os_remount_dir)) {
+	return &os_remount_dir(@_);
+	}
+else {
+	local $err = &unmount_dir(@_);
+	return $err if ($err);
+	return &mount_dir(@_);
+	}
+}
+
+# filesystem_for_dir(dir)
+# Give a directory, returns the details filesystem it is on (dir, device,
+# type, options)
+sub filesystem_for_dir
+{
+local @stdir = stat($_[0]);
+foreach my $m (&list_mounted()) {
+	local @stm = stat($m->[0]);
+	if ($stm[0] == $stdir[0]) {
+		# Save device number!
+		return @$m;
+		}
+	}
+return ( );
+}
+
 1;
 
