@@ -141,13 +141,15 @@ foreach (@autotab) {
 sub list_mounted
 {
 local(@rv, @p, $_, $i, $r);
-foreach (split(/\n/, `/usr/sbin/mount`)) {
+&open_execute_command(MOUNT, "/usr/sbin/mount", 1, 1);
+while(<MOUNT>) {
 	if (/^(\S+)\s+on\s+(\S+)\s+type\s+(\S+)\s+\((.*)\)/) {
 		local $opts = $3;
 		$opts =~ s/\s+//g;
 		push(@rv, [ $2, $1, $3, $opts ]);
 		}
 	}
+close(MOUNT);
 return @rv;
 }
 
@@ -161,7 +163,7 @@ sub mount_dir
 local($out, $opts);
 if ($_[0] eq "swap") {
 	# Adding a swap device
-	$out = `swap -a $_[1] 2>&1`;
+	$out = &backquote_logged("swap -a $_[1] 2>&1");
 	if ($?) { return $out; }
 	}
 else {
@@ -177,7 +179,7 @@ else {
 				return "The directory $options{cachedir} ".
 				       "already exists. Delete it";
 				}
-			$out = `cfsadmin -c $options{cachedir} 2>&1`;
+			$out = &backquote_logged("cfsadmin -c $options{cachedir} 2>&1");
 			if ($?) { return $out; }
 			}
 		}
@@ -206,13 +208,13 @@ else {
 		 (defined($options{'readonly'}) ? "-w " : "").
 		 (defined($options{'attr'}) ? "-e " : "");
 		local $rtemp = &transname();
-		$rv = system("rumba \"$shar\" $_[0] $opts >$rtemp 2>&1 </dev/null");
+		$rv = &system_logged("rumba \"$shar\" $_[0] $opts >$rtemp 2>&1 </dev/null");
 		$out = `cat $rtemp`; unlink($rtemp);
 		if ($rv) { return "<pre>$out</pre> : rumba \"$shar\" $_[0] $opts"; }
 		}
 	else {
 		$opts = $_[3] eq "-" ? "" : "-o \"$_[3]\"";
-		$out = `mount -F $_[2] $opts -- $_[1] $_[0] 2>&1`;
+		$out = &backquote_logged("mount -F $_[2] $opts -- $_[1] $_[0] 2>&1");
 		if ($?) { return $out; }
 		}
 	}
@@ -226,20 +228,25 @@ return 0;
 sub unmount_dir
 {
 if ($_[0] eq "swap") {
-	$out = `swap -d $_[1] 2>&1`;
+	$out = &backquote_logged("swap -d $_[1] 2>&1");
 	}
 elsif ($_[2] eq "rumba") {
 	# kill the process (if nobody is in the directory)
 	$dir = $_[0];
-	if (`fuser -c $_[0] 2>/dev/null` =~ /\d/) { return "$_[0] is busy"; }
-	if (`cat /etc/mnttab` =~ /rumba-(\d+)\s+$dir\s+nfs/) {
+	if (&backquote_command("fuser -c $_[0] 2>/dev/null", 1) =~ /\d/) {
+		return "$_[0] is busy";
+		}
+	if (&backquote_command("cat /etc/mnttab", 1) =~
+	    /rumba-(\d+)\s+$dir\s+nfs/) {
 		&kill_logged('TERM', $1) || return "Failed to kill rumba";
 		}
-	else { return "Failed to find rumba pid"; }
+	else {
+		return "Failed to find rumba pid";
+		}
 	sleep(1);
 	}
 else {
-	$out = `umount $_[0] 2>&1`;
+	$out = &backquote_logged("umount $_[0] 2>&1");
 	}
 if ($?) { return $out; }
 return 0;
@@ -255,8 +262,11 @@ if (&get_mounted($_[1], "*") < 0) { return (); }
 if ($_[0] eq "fd" || $_[0] eq "proc" || $_[0] eq "swap" || $_[0] eq "autofs") {
 	return ();
 	}
-`df -k $_[1]` =~ /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/;
-return ($1, $2);
+if (&backquote_command("df -k $_[1]", 1) =~
+    /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/) {
+	return ($1, $2);
+	}
+return ( );
 }
 
 
@@ -576,11 +586,13 @@ if ($_[0] eq "nfs") {
 	print "<td nowrap><select name=proto>\n";
 	printf "<option value=\"\" %s> Default\n",
 		defined($options{"proto"}) ? "" : "selected";
-	foreach (split(/\n/, `cat /etc/netconfig`)) {
+	&open_tempfile(NETCONFIG, "/etc/netconfig");
+	while(<NETCONFIG>) {
 		if (!/^([A-z0-9\_\-]+)\s/) { next; }
 		printf "<option value=\"$1\" %s> $1\n",
 			$options{"proto"} eq $1 ? "selected" : "";
 		}
+	close(NETCONFIG);
 	print "</select></td>\n";
 
 	print "<td><b>NFS Port</b></td>\n";
@@ -946,7 +958,7 @@ if ($_[0] eq "nfs") {
 	if ($in{nfs_host} !~ /^\S+$/) {
 		&error("'$in{nfs_host}' is not a valid hostname");
 		}
-	$out = `dfshares '$in{nfs_host}' 2>&1`;
+	$out = &backquote_command("dfshares '$in{nfs_host}' 2>&1");
 	if ($out =~ /Unknown host/) {
 		&error("The host '$in{nfs_host}' does not exist");
 		}
@@ -968,8 +980,8 @@ if ($_[0] eq "nfs") {
 		       "<pre>$dirlist</pre>");
 		}
 	$temp = &transname();
-	mkdir($temp, 0755);
-	$mout = `mount $in{nfs_host}:$in{nfs_dir} $temp 2>&1`;
+	&make_dir($temp, 0755);
+	$mout = &backquote_command("mount $in{nfs_host}:$in{nfs_dir} $temp 2>&1");
 	if ($mout =~ /No such file or directory/) {
 		rmdir($temp);
 		&error("The directory '$in{nfs_dir}' does not exist on the ".
@@ -986,8 +998,8 @@ if ($_[0] eq "nfs") {
 		&error("NFS Error - $mout");
 		}
 	# It worked! unmount
-	`umount $temp`;
-	rmdir($temp);
+	&execute_command("umount $temp");
+	&unlink_file($temp);
 	return "$in{nfs_host}:$in{nfs_dir}";
 	}
 elsif ($_[0] eq "ufs") {
@@ -1130,7 +1142,7 @@ local($out, $part, $found);
 # Check if the device/partition actually exists
 if ($_[0] =~ /^\/dev\/dsk\/c(.)t(.)d(.)s(.)$/) {
 	# mounting a normal scsi device..
-	$out = `prtvtoc -h $_[0] 2>&1`;
+	$out = &backquote_command("prtvtoc -h $_[0] 2>&1");
 	if ($out =~ /No such file or directory|No such device or address/) {
 		&error("The SCSI target for '$_[0]' does not exist");
 		}
@@ -1147,7 +1159,7 @@ if ($_[0] =~ /^\/dev\/dsk\/c(.)t(.)d(.)s(.)$/) {
 	}
 elsif ($_[0] =~ /^\/dev\/md\/dsk\/d(.)$/) {
 	# mounting a multi-disk (raid) device..
-	$out = `prtvtoc -h $_[0] 2>&1`;
+	$out = &backquote_command("prtvtoc -h $_[0] 2>&1");
 	if ($out =~ /No such file or directory|No such device or address/) {
 		&error("The RAID device for '$_[0]' does not exist");
 		}
@@ -1169,7 +1181,7 @@ else {
 	}
 
 # Check the filesystem type
-$out = `fstyp $_[0] 2>&1`;
+$out = &backquote_command("fstyp $_[0] 2>&1");
 if ($out =~ /^([A-z0-9]+)\n$/) {
 	if ($1 eq $_[1]) { return; }
 	else {
@@ -1416,9 +1428,9 @@ return @rv ? join(',' , @rv) : "-";
 sub create_swap
 {
 local($out);
-$out = `mkfile $_[1]$_[2] $_[0] 2>&1`;
+$out = &backquote_logged("mkfile $_[1]$_[2] $_[0] 2>&1");
 if ($?) {
-	unlink($_[0]);
+	&unlink_file($_[0]);
 	return "mkfile failed : $out";
 	}
 return 0;
@@ -1432,7 +1444,7 @@ sub exports_list
 {
 local($dref, $cref, $out, $_);
 $dref = $_[1]; $cref = $_[2];
-$out = `showmount -e $_[0] 2>&1`;
+$out = &backquote_command("showmount -e ".quotemeta($_[0])." 2>&1", 1);
 if ($?) { return $out; }
 foreach (split(/\n/, $out)) {
 	if (/^(\/\S*)\s+(.*)$/) {
@@ -1447,7 +1459,7 @@ return undef;
 sub broadcast_addr
 {
 local($out);
-$out = `ifconfig -a 2>&1`;
+$out = &backquote_command("ifconfig -a 2>&1", 1);
 if ($out =~ /broadcast\s+(\S+)/) { return $1; }
 return "255.255.255.255";
 }
