@@ -162,10 +162,13 @@ sub list_mounted
 {
 return @list_mounted_cache if (@list_mounted_cache);
 local(@rv, @p, $_, $i, $r);
-foreach (split(/\n/, `swap -l 2>/dev/null`)) {
+&open_execute_command(SWAP, "swap -l 2>/dev/null", 1, 1);
+while(<SWAP>) {
 	if (/^(\/\S+)\s+/) { push(@rv, [ "swap", $1, "swap", "-" ]); }
 	}
-foreach (split(/\n/, `cat /etc/mnttab`)) {
+close(SWAP);
+&open_tempfile(MNTTAB, "/etc/mnttab");
+while(<MNTTAB>) {
 	s/#.*$//g; if (!/\S/) { next; }
 	@p = split(/\s+/, $_);
 	if ($p[0] =~ /:vold/) { next; }
@@ -174,7 +177,7 @@ foreach (split(/\n/, `cat /etc/mnttab`)) {
 		local($args, $ps); $p[3] = "pid=$2";
 		$ps = (-x "/usr/ucb/ps") ? "/usr/ucb/ps auwwwwx $2"
 				 	 : "ps -o args -f $2";
-		`$ps` =~ /(rumba|shlight)\s+\/\/([^\/]+)\/(.*\S)\s+(\/\S+)(.*)/ || next;
+		&backquote_command($ps, 1) =~ /(rumba|shlight)\s+\/\/([^\/]+)\/(.*\S)\s+(\/\S+)(.*)/ || next;
 		$serv = $2; $shar = $3; $p[2] = "rumba"; $args = $5;
 		if ($args =~ /\s+-s\s+(\S+)/ && $1 ne $serv) {
 			$p[0] = "\\\\$1\\$shar";
@@ -196,6 +199,7 @@ foreach (split(/\n/, `cat /etc/mnttab`)) {
 	else { $p[3] = join(',' , (grep {!/^dev=/} split(/,/ , $p[3]))); }
 	push(@rv, [ $p[1], $p[0], $p[2], $p[3] ]);
 	}
+&close_tempfile(MNTTAB);
 foreach $r (@rv) {
 	if ($r->[2] eq "cachefs" && $r->[1] =~ /\.cfs_mnt_points/) {
 		# Oh no.. a caching filesystem mount. Fiddle things so that
@@ -269,7 +273,7 @@ else {
 		 (defined($options{'readonly'}) ? "-w " : "").
 		 (defined($options{'attr'}) ? "-e " : "");
 		local $rtemp = &transname();
-		$rv = system("rumba \"$shar\" $_[0] $opts >$rtemp 2>&1 </dev/null");
+		$rv = &system_logged("rumba \"$shar\" $_[0] $opts >$rtemp 2>&1 </dev/null");
 		$out = `cat $rtemp`; unlink($rtemp);
 		if ($rv) { return "<pre>$out</pre> : rumba \"$shar\" $_[0] $opts"; }
 		}
@@ -295,12 +299,16 @@ if ($_[0] eq "swap") {
 elsif ($_[2] eq "rumba") {
 	# kill the process (if nobody is in the directory)
 	$dir = $_[0];
-	if (`fuser -c $_[0] 2>/dev/null` =~ /\d/)
-		{ return &text('solaris_ebusy', $_[0]); }
-	if (`cat /etc/mnttab` =~ /(rumba|shlight)-(\d+)\s+$dir\s+nfs/) {
+	if (&backquote_command("fuser -c $_[0] 2>/dev/null", 1) =~ /\d/) {
+		return &text('solaris_ebusy', $_[0]);
+		}
+	if (&backquote_command("cat /etc/mnttab", 1) =~
+	    /(rumba|shlight)-(\d+)\s+$dir\s+nfs/) {
 		&kill_logged('TERM', $2) || return $text{'solaris_ekill'};
 		}
-	else { return $text{'solaris_epid'}; }
+	else {
+		return $text{'solaris_epid'};
+		}
 	sleep(1);
 	}
 else {
@@ -321,8 +329,11 @@ if (&get_mounted($_[1], "*") < 0) { return (); }
 if ($_[0] eq "fd" || $_[0] eq "proc" || $_[0] eq "swap" || $_[0] eq "autofs") {
 	return ();
 	}
-`df -k $_[1]` =~ /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/;
-return ($1, $2);
+if (&backquote_command("df -k $_[1]", 1) =~
+    /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/) {
+	return ($1, $2);
+	}
+return ( );
 }
 
 
@@ -680,11 +691,13 @@ if ($_[0] eq "nfs") {
 	print "<td nowrap><select name=proto>\n";
 	printf "<option value=\"\" %s> $text{'default'}\n",
 		defined($options{"proto"}) ? "" : "selected";
-	foreach (split(/\n/, `cat /etc/netconfig`)) {
+	&open_tempfile(NETCONFIG, "/etc/netconfig");
+	while(<NETCONFIG>) {
 		if (!/^([A-z0-9\_\-]+)\s/) { next; }
 		printf "<option value=\"$1\" %s> $1\n",
 			$options{"proto"} eq $1 ? "selected" : "";
 		}
+	&close_tempfile(NETCONFIG);
 	print "</select></td>\n";
 
 	print "<td><b>$text{'solaris_port'}</b></td>\n";
@@ -862,8 +875,18 @@ if ($_[0] eq "pcfs") {
 		}
 	}
 if ($_[0] eq "lofs") {
-	# No options as far as I know
-	print "<tr> <td><i>$text{'solaris_noopts'}</i></td> </tr>\n";
+	# LOFS is a loopback filesystem
+	print "<tr> <td><b>$text{'solaris_ro'}</b></td>\n";
+	printf "<td nowrap><input type=radio name=ufs_ro value=1 %s> $text{'yes'}\n",
+		defined($options{"ro"}) ? "checked" : "";
+	printf "<input type=radio name=ufs_ro value=0 %s> $text{'no'}</td>\n",
+		defined($options{"ro"}) ? "" : "checked";
+
+	print "<td><b>$text{'solaris_nosuid'}</b></td>\n";
+	printf "<td nowrap><input type=radio name=ufs_nosuid value=1 %s> $text{'yes'}\n",
+		defined($options{"nosuid"}) ? "checked" : "";
+	printf "<input type=radio name=ufs_nosuid value=0 %s> $text{'no'}</td> </tr>\n",
+		defined($options{"nosuid"}) ? "" : "checked";
 	}
 if ($_[0] eq "tmpfs") {
 	# Solaris tmpfs (virtual memory) filesystem.
@@ -1099,7 +1122,7 @@ if ($_[0] eq "nfs") {
 	if ($in{nfs_host} !~ /^\S+$/) {
 		&error(&text('solaris_ehost', $in{'nfs_host'}));
 		}
-	$out = `dfshares '$in{nfs_host}' 2>&1`;
+	$out = &backquote_command("dfshares '$in{nfs_host}' 2>&1");
 	if ($out =~ /Unknown host/) {
 		&error(&text('solaris_ehost2', $in{'nfs_host'}));
 		}
@@ -1119,8 +1142,8 @@ if ($_[0] eq "nfs") {
 			     $in{'nfs_host'}, "<pre>$dirlist</pre>"));
 		}
 	$temp = &transname();
-	mkdir($temp, 0755);
-	$mout = `mount $in{nfs_host}:$in{nfs_dir} $temp 2>&1`;
+	&make_dir($temp, 0755);
+	$mout = &backquote_command("mount $in{nfs_host}:$in{nfs_dir} $temp 2>&1");
 	if ($mout =~ /No such file or directory/) {
 		rmdir($temp);
 		&error(&text('solaris_enfsdir', $in{'nfs_dir'},
@@ -1135,8 +1158,8 @@ if ($_[0] eq "nfs") {
 		&error(&text('solaris_enfsmount', "<tt>$mout</tt>"));
 		}
 	# It worked! unmount
-	`umount $temp`;
-	rmdir($temp);
+	&execute_command("umount $temp");
+	&unlink_file($temp);
 	return "$in{nfs_host}:$in{nfs_dir}";
 	}
 elsif ($_[0] eq "ufs" || $_[0] eq "pcfs") {
@@ -1286,7 +1309,7 @@ local($out, $part, $found);
 # Check if the device/partition actually exists
 if ($_[0] =~ /^\/dev\/dsk\/c(.)t(.)d(.)s(.)$/) {
 	# mounting a normal scsi device..
-	$out = `prtvtoc -h $_[0] 2>&1`;
+	$out = &backquote_command("prtvtoc -h $_[0] 2>&1");
 	if ($out =~ /No such file or directory|No such device or address/) {
 		&error(&text('solaris_etarget2', $_[0]));
 		}
@@ -1303,7 +1326,7 @@ if ($_[0] =~ /^\/dev\/dsk\/c(.)t(.)d(.)s(.)$/) {
 	}
 elsif ($_[0] =~ /^\/dev\/md\/dsk\/d(.)$/) {
 	# mounting a multi-disk (raid) device..
-	$out = `prtvtoc -h $_[0] 2>&1`;
+	$out = &backquote_command("prtvtoc -h $_[0] 2>&1");
 	if ($out =~ /No such file or directory|No such device or address/) {
 		&error(&text('solaris_eraid2', $_[0]));
 		}
@@ -1325,7 +1348,7 @@ else {
 	}
 
 # Check the filesystem type
-$out = `fstyp $_[0] 2>&1`;
+$out = &backquote_command("fstyp $_[0] 2>&1");
 if ($out =~ /^([A-Za-z0-9]+)\n$/) {
 	if ($1 eq $_[1]) { return; }
 	else {
@@ -1462,7 +1485,18 @@ elsif ($_[0] eq "ufs") {
 		}
 	}
 elsif ($_[0] eq "lofs") {
-	# Loopback has no options to parse
+	# LOFS has a few options
+	if ($in{'nfs_ro'}) {
+		# Read-only
+		$options{"ro"} = ""; delete($options{"rw"});
+		}
+	else {
+		# Read-write
+		$options{"rw"} = ""; delete($options{"ro"});
+		}
+
+	delete($options{"nosuid"});
+	if ($in{ufs_nosuid}) { $options{"nosuid"} = ""; }
 	}
 elsif ($_[0] eq "swap") {
 	# Swap has no options to parse
@@ -1618,9 +1652,9 @@ return @rv ? join(',' , @rv) : "-";
 sub create_swap
 {
 local($out);
-$out = `mkfile $_[1]$_[2] $_[0] 2>&1`;
+$out = &backquote_logged("mkfile $_[1]$_[2] $_[0] 2>&1");
 if ($?) {
-	unlink($_[0]);
+	&unlink_file($_[0]);
 	return "mkfile failed : $out";
 	}
 return 0;
@@ -1634,7 +1668,7 @@ sub exports_list
 {
 local($dref, $cref, $out, $_);
 $dref = $_[1]; $cref = $_[2];
-$out = `showmount -e $_[0] 2>&1`;
+$out = &backquote_command("showmount -e ".quotemeta($_[0])." 2>&1", 1);
 if ($?) { return $out; }
 foreach (split(/\n/, $out)) {
 	if (/^(\/\S*)\s+(.*)$/) {
@@ -1649,7 +1683,7 @@ return undef;
 sub broadcast_addr
 {
 local($out);
-$out = `ifconfig -a 2>&1`;
+$out = &backquote_command("ifconfig -a 2>&1", 1);
 if ($out =~ /broadcast\s+(\S+)/) { return $1; }
 return "255.255.255.255";
 }
