@@ -153,10 +153,13 @@ foreach (@autotab) {
 sub list_mounted
 {
 local(@rv, @p, $_, $i, $r);
-foreach (split(/\n/, `swap -l 2>/dev/null`)) {
+&open_execute_command(SWAP, "swap -l 2>/dev/null", 1, 1);
+while(<SWAP>) {
 	if (/^(\/\S+)\s+/) { push(@rv, [ "swap", $1, "swap", "-" ]); }
 	}
-foreach (split(/\n/, `cat /etc/mnttab`)) {
+close(SWAP);
+&open_tempfile(MNTTAB, "/etc/mnttab");
+while(<MNTTAB>) {
 	s/#.*$//g; if (!/\S/) { next; }
 	@p = split(/\s+/, $_);
 	if ($p[0] =~ /:vold/) { next; }
@@ -165,7 +168,8 @@ foreach (split(/\n/, `cat /etc/mnttab`)) {
 		local($args, $ps); $p[3] = "pid=$1";
 		$ps = (-x "/usr/ucb/ps") ? "/usr/ucb/ps auwwwwx $1"
 				 	 : "ps -o args -f $1";
-		`$ps` =~ /rumba\s+\/\/([^\/]+)\/(.*\S)\s+(\/\S+)(.*)/ || next;
+		&backquote_command($ps, 1) =~
+			/rumba\s+\/\/([^\/]+)\/(.*\S)\s+(\/\S+)(.*)/ || next;
 		$serv = $1; $shar = $2; $p[2] = "rumba"; $args = $4;
 		if ($args =~ /\s+-s\s+(\S+)/ && $1 ne $serv) {
 			$p[0] = "\\\\$1\\$shar";
@@ -187,6 +191,7 @@ foreach (split(/\n/, `cat /etc/mnttab`)) {
 	else { $p[3] = join(',' , (grep {!/^dev=/} split(/,/ , $p[3]))); }
 	push(@rv, [ $p[1], $p[0], $p[2], $p[3] ]);
 	}
+&close_tempfile(MNTTAB);
 foreach $r (@rv) {
 	if ($r->[2] eq "cachefs" && $r->[1] =~ /\.cfs_mnt_points/) {
 		# Oh no.. a caching filesystem mount. Fiddle things so that
@@ -259,7 +264,7 @@ else {
 		 (defined($options{'readonly'}) ? "-w " : "").
 		 (defined($options{'attr'}) ? "-e " : "");
 		local $rtemp = &transname();
-		$rv = system("rumba \"$shar\" $_[0] $opts >$rtemp 2>&1 </dev/null");
+		$rv = &system_logged("rumba \"$shar\" $_[0] $opts >$rtemp 2>&1 </dev/null");
 		$out = `cat $rtemp`; unlink($rtemp);
 		if ($rv) { return "<pre>$out</pre> : rumba \"$shar\" $_[0] $opts"; }
 		}
@@ -284,11 +289,16 @@ if ($_[0] eq "swap") {
 elsif ($_[2] eq "rumba") {
 	# kill the process (if nobody is in the directory)
 	$dir = $_[0];
-	if (`fuser -c $_[0] 2>/dev/null` =~ /\d/) { return "$_[0] is busy"; }
-	if (`cat /etc/mnttab` =~ /rumba-(\d+)\s+$dir\s+nfs/) {
+	if (&backquote_command("fuser -c $_[0] 2>/dev/null") =~ /\d/) {
+		return "$_[0] is busy";
+		}
+	if (&backquote_command("cat /etc/mnttab") =~
+	    /rumba-(\d+)\s+$dir\s+nfs/) {
 		&kill_logged('TERM', $1) || return "Failed to kill rumba";
 		}
-	else { return "Failed to find rumba pid"; }
+	else {
+		return "Failed to find rumba pid";
+		}
 	sleep(1);
 	}
 else {
@@ -308,8 +318,11 @@ if (&get_mounted($_[1], "*") < 0) { return (); }
 if ($_[0] eq "fd" || $_[0] eq "proc" || $_[0] eq "swap" || $_[0] eq "autofs") {
 	return ();
 	}
-`df -k $_[1]` =~ /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/;
-return ($1, $2);
+if (&backquote_command("df -k $_[1]", 1) =~
+    /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/) {
+	return ($1, $2);
+	}
+return ( );
 }
 
 
@@ -690,11 +703,13 @@ if ($_[0] eq "nfs") {
 	print "<td nowrap><select name=proto>\n";
 	printf "<option value=\"\" %s> Default\n",
 		defined($options{"proto"}) ? "" : "selected";
-	foreach (split(/\n/, `cat /etc/netconfig`)) {
+	&open_tempfile(NETCONFIG, "/etc/netconfig");
+	while(<NETCONFIG>) {
 		if (!/^([A-z0-9\_\-]+)\s/) { next; }
 		printf "<option value=\"$1\" %s> $1\n",
 			$options{"proto"} eq $1 ? "selected" : "";
 		}
+	&close_tempfile(NETCONFIG);
 	print "</select></td>\n";
 
 	print "<td><b>NFS Port</b></td>\n";
@@ -1097,7 +1112,7 @@ if ($_[0] eq "nfs") {
 	if ($in{nfs_host} !~ /^\S+$/) {
 		&error("'$in{nfs_host}' is not a valid hostname");
 		}
-	$out = `dfshares '$in{nfs_host}' 2>&1`;
+	$out = &backquote_command("dfshares '$in{nfs_host}' 2>&1");
 	if ($out =~ /Unknown host/) {
 		&error("The host '$in{nfs_host}' does not exist");
 		}
@@ -1119,8 +1134,8 @@ if ($_[0] eq "nfs") {
 		       "<pre>$dirlist</pre>");
 		}
 	$temp = &transname();
-	mkdir($temp, 0755);
-	$mout = `mount $in{nfs_host}:$in{nfs_dir} $temp 2>&1`;
+	&make_dir($temp, 0755);
+	$mout = &backquote_command("mount $in{nfs_host}:$in{nfs_dir} $temp 2>&1");
 	if ($mout =~ /No such file or directory/) {
 		rmdir($temp);
 		&error("The directory '$in{nfs_dir}' does not exist on the ".
@@ -1137,8 +1152,8 @@ if ($_[0] eq "nfs") {
 		&error("NFS Error - $mout");
 		}
 	# It worked! unmount
-	`umount $temp`;
-	rmdir($temp);
+	&execute_command("umount $temp");
+	&unlink_file($temp);
 	return "$in{nfs_host}:$in{nfs_dir}";
 	}
 elsif ($_[0] eq "ufs") {
@@ -1310,7 +1325,7 @@ local($out, $part, $found);
 # Check if the device/partition actually exists
 if ($_[0] =~ /^\/dev\/dsk\/c(.)t(.)d(.)s(.)$/) {
 	# mounting a normal scsi device..
-	$out = `prtvtoc -h $_[0] 2>&1`;
+	$out = &backquote_command("prtvtoc -h $_[0] 2>&1");
 	if ($out =~ /No such file or directory|No such device or address/) {
 		&error("The SCSI target for '$_[0]' does not exist");
 		}
@@ -1327,7 +1342,7 @@ if ($_[0] =~ /^\/dev\/dsk\/c(.)t(.)d(.)s(.)$/) {
 	}
 elsif ($_[0] =~ /^\/dev\/md\/dsk\/d(.)$/) {
 	# mounting a multi-disk (raid) device..
-	$out = `prtvtoc -h $_[0] 2>&1`;
+	$out = &backquote_command("prtvtoc -h $_[0] 2>&1");
 	if ($out =~ /No such file or directory|No such device or address/) {
 		&error("The RAID device for '$_[0]' does not exist");
 		}
@@ -1349,7 +1364,7 @@ else {
 	}
 
 # Check the filesystem type
-$out = `fstyp $_[0] 2>&1`;
+$out = &backquote_command("fstyp $_[0] 2>&1");
 if ($out =~ /^([A-z0-9]+)\n$/) {
 	if ($1 eq $_[1]) { return; }
 	else {
@@ -1641,9 +1656,9 @@ return @rv ? join(',' , @rv) : "-";
 sub create_swap
 {
 local($out);
-$out = `mkfile $_[1]$_[2] $_[0] 2>&1`;
+$out = &backquote_logged("mkfile $_[1]$_[2] $_[0] 2>&1");
 if ($?) {
-	unlink($_[0]);
+	&unlink_file($_[0]);
 	return "mkfile failed : $out";
 	}
 return 0;
@@ -1657,7 +1672,7 @@ sub exports_list
 {
 local($dref, $cref, $out, $_);
 $dref = $_[1]; $cref = $_[2];
-$out = `showmount -e $_[0] 2>&1`;
+$out = &backquote_command("showmount -e ".quotemeta($_[0])." 2>&1", 1);
 if ($?) { return $out; }
 foreach (split(/\n/, $out)) {
 	if (/^(\/\S*)\s+(.*)$/) {
@@ -1672,7 +1687,7 @@ return undef;
 sub broadcast_addr
 {
 local($out);
-$out = `ifconfig -a 2>&1`;
+$out = &backquote_command("ifconfig -a 2>&1", 1);
 if ($out =~ /broadcast\s+(\S+)/) { return $1; }
 return "255.255.255.255";
 }
