@@ -224,14 +224,16 @@ if (@_ > 0) {
     print $_[7] if ($_[7]);
     if ($gconfig{'sysinfo'} == 0 && $remote_user) {
         print "<SCRIPT LANGUAGE=\"JavaScript\">\n";
-        printf
-        "defaultStatus=\"%s%s logged into %s %s on %s (%s %s)\";\n",
-            $ENV{'ANONYMOUS_USER'} ? "Anonymous user" : $remote_user,
-            $ENV{'SSL_USER'} ? " (SSL certified)" :
-            $ENV{'LOCAL_USER'} ? " (Local user)" : "",
-	    $text{'programname'},
-            &get_webmin_version(), &get_display_hostname(),
-            $os_type, $os_version;
+	print "defaultStatus=\"".&text('header_statusmsg',
+		    ($ENV{'ANONYMOUS_USER'} ? "Anonymous user"
+					   : $remote_user).
+		    ($ENV{'SSL_USER'} ? " (SSL certified)" :
+		     $ENV{'LOCAL_USER'} ? " (Local user)" : ""),
+		    $text{'programname'},
+		    &get_webmin_version(),
+		    &get_display_hostname(),
+		    $os_type.($os_version eq "*" ? "" :" $os_version")).
+		"\";\n";
         print "</SCRIPT>\n";
         }
     }
@@ -240,11 +242,13 @@ if (@_ > 0) {
 	if (!defined(@msc_modules));
 
 print "</head>\n";
+local $dir = $current_lang_info->{'dir'} ? "dir=\"$current_lang_info->{'dir'}\""
+					 : "";
 if ($theme_no_table) {
-	print '<body bgcolor=#6696bc link=#000000 vlink=#000000 text=#000000 leftmargin="0" topmargin="0" marginwidth="0" marginheight="0" '.$_[8].'>';
+	print '<body bgcolor=#6696bc link=#000000 vlink=#000000 text=#000000 leftmargin="0" topmargin="0" marginwidth="0" marginheight="0" '.$dir.' '.$_[8].'>';
 	}
 else {
-	print '<body bgcolor=#6696bc link=#000000 vlink=#000000 text=#000000 leftmargin="0" topmargin="0" marginwidth="0" marginheight="0" '.$_[8].'>';
+	print '<body bgcolor=#6696bc link=#000000 vlink=#000000 text=#000000 leftmargin="0" topmargin="0" marginwidth="0" marginheight="0" '.$dir.' '.$_[8].'>';
 	}
 
 if ($remote_user && @_ > 1) {
@@ -676,7 +680,24 @@ $main::printed_unbuffered_header = 1;
 # Returns HTML for a section heading
 sub theme_ui_subheading
 {
-return "<font size=+1>",join("", @_),"</font><br>\n";
+return "<font size=+1>".join("", @_)."</font><br>\n";
+}
+
+# theme_popup_header(title, headstuff, bodystuff)
+sub theme_popup_header
+{
+print "<!doctype html public \"-//W3C//DTD HTML 3.2 Final//EN\">\n";
+print "<html>\n";
+print "<title>$_[0]</title>\n";
+print $_[1];
+print "</head>\n";
+local $margin = $_[0] ? 2 : 0;
+print "<body bgcolor=#6696bc link=#000000 vlink=#000000 text=#000000 leftmargin=$margin topmargin=$margin marginwidth=$margin marginheight=$margin $_[2]>\n";
+}
+
+sub theme_popup_footer
+{
+print "</body></html>\n";
 }
 
 1;
