@@ -77,9 +77,7 @@ sub delete_interface
 {
 local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
 				       : $_[0]->{'name'};
-&lock_file("$net_scripts_dir/ifcfg-$name");
-unlink(&translate_filename("$net_scripts_dir/ifcfg-$name"));
-&unlock_file("$net_scripts_dir/ifcfg-$name");
+&unlink_logged("$net_scripts_dir/ifcfg-$name");
 }
 
 # can_edit(what)
@@ -183,11 +181,12 @@ local %conf;
 &read_file($network_config, \%conf);
 if ($in{'forward'}) { $conf{'IPFORWARDING'} = 'yes'; }
 else { delete($conf{'IPFORWARDING'}); }
+local %ifcs = map { $_->{'fullname'}, 1 } &all_interfaces();
 
 if (!$in{'gateway_def'}) {
 	gethostbyname($in{'gateway'}) ||
 		&error(&text('routes_edefault', $in{'gateway'}));
-	-r "$net_scripts_dir/ifcfg-$in{'gatewaydev'}" ||
+	$ifcs{$in{'gatewaydev'}} ||
 		&error(&text('routes_edevice', $in{'gatewaydev'}));
 	}
 
@@ -225,7 +224,7 @@ sub apply_network
 # Calls an OS-specific function to make a boot-time interface active
 sub apply_interface
 {
-local $out = &backquote_logged("cd ; ifup '$_[0]->{'fullname'}' 2>&1 </dev/null");
+local $out = &backquote_logged("cd / ; ifup '$_[0]->{'fullname'}' 2>&1 </dev/null");
 return $? ? $out : undef;
 }
 
