@@ -6,7 +6,7 @@
 sub active_interfaces
   {
       local(@rv, @lines, $l);
-      &open_execute_command(IFC, "ifconfig -A", 1);
+      &open_execute_command(IFC, "ifconfig -A", 1, 1);
       while(<IFC>) {
 	  s/\r|\n//g;
 	  if (/^\S+:/) { push(@lines, $_); }
@@ -187,7 +187,9 @@ sub save_interface
     
     &lock_file("/etc/hostname.$if");
     if ( $_[0]->{'dhcp'} ){ 
-      system("echo dhcp > /etc/hostname.$if"); 
+      &open_tempfile( IF_FILE, ">/etc/hostname.".$if );
+      &print_tempfile(IF_FILE, "dhcp\n");
+      &close_tempfile( IF_FILE );
       &unlock_file("/etc/hostname.$if");
       return;
     }
@@ -269,7 +271,7 @@ sub delete_interface
 	# check if we deleted the only entry in the file
 	# if so delete the file (otherwise dhcp will be used for the interface)
 	if( $found == 1 ) {
-	    unlink (&translate_filename("/etc/hostname.".$if));
+	    &unlink_logged("/etc/hostname.".$if);
 	}
     } 
     &unlock_file("/etc/hostname.$if");
@@ -492,13 +494,18 @@ sub parse_routing
 
       &read_routing;
 
+      &lock_file("/etc/mygate");
       if ( $in{'defr_def'} && -f "/etc/mygate" ) {
-	  unlink(&translate_filename("/etc/mygate"));
+	  &unlink_file("/etc/mygate");
       }
       else {
-	  if( $in{'defr'} ne $defr )
-	    { system("echo ".$in{'defr'}." > /etc/mygate"); }
+	  if( $in{'defr'} ne $defr ) {
+		&open_tempfile(MYGATE, ">/etc/mygate");
+		&print_tempfile(MYGATE, $in{'defr'},"\n");
+		&close_tempfile(MYDATE);
+	  }
       }
+      &unlock_file("/etc/mygate");
       
       if( $in{'gw'} ne $gw ) {
 	  &set_line( "/etc/sysctl.conf", "^net.inet.ip.forwarding", "net.inet.ip.forwarding=$in{'gw'}" );
