@@ -25,7 +25,7 @@ push(@rv, { 'up' => 1,
 
 # Look in inet1.conf file for master interfaces
 local $lnum = 0;
-open(CONF, $inet_conf);
+&open_readfile(CONF, $inet_conf);
 while(<CONF>) {
 	if (/^\s*IPADDR\[(\d+)\]\s*=\s*"(.*)"/) {
 		push(@rv, { 'up' => 1,
@@ -74,7 +74,7 @@ for($i=0; $i<@rv; $i++) {
 
 # Read extra init script for virtual interfaces
 local $lnum = 0;
-open(IFACES, $interfaces_file);
+&open_readfile(IFACES, $interfaces_file);
 while(<IFACES>) {
 	s/\r|\n//g;
 	if (/^(#*)\s*(\S*ifconfig)\s+(\S+)\s+(\S+)(\s+netmask\s+(\S+))?(\s+broadcast\s+(\S+))?(\s+mtu\s+(\d+))?\s+up$/) {
@@ -251,128 +251,6 @@ sub valid_boot_address
 return &check_ipaddress($_[0]);
 }
 
-# get_dns_config()
-# Returns a hashtable containing keys nameserver, domain, search & order
-sub get_dns_config
-{
-local $dns;
-open(RESOLV, "/etc/resolv.conf");
-while(<RESOLV>) {
-	s/\r|\n//g;
-	s/#.*$//;
-	if (/nameserver\s+(.*)/) {
-		push(@{$dns->{'nameserver'}}, split(/\s+/, $1));
-		}
-	elsif (/domain\s+(\S+)/) {
-		$dns->{'domain'} = [ $1 ];
-		}
-	elsif (/search\s+(.*)/) {
-		$dns->{'domain'} = [ split(/\s+/, $1) ];
-		}
-	}
-close(RESOLV);
-open(SWITCH, "/etc/nsswitch.conf");
-while(<SWITCH>) {
-	s/\r|\n//g;
-	if (/^\s*hosts:\s+(.*)/) {
-		$dns->{'order'} = $1;
-		}
-	}
-close(SWITCH);
-$dns->{'files'} = [ "/etc/resolv.conf", "/etc/nsswitch.conf" ];
-return $dns;
-}
-
-# save_dns_config(&config)
-# Writes out the resolv.conf and nsswitch.conf files
-sub save_dns_config
-{
-&lock_file("/etc/resolv.conf");
-open(RESOLV, "/etc/resolv.conf");
-local @resolv = <RESOLV>;
-close(RESOLV);
-open(RESOLV, ">/etc/resolv.conf");
-foreach (@{$_[0]->{'nameserver'}}) {
-	print RESOLV "nameserver $_\n";
-	}
-if ($_[0]->{'domain'}) {
-	if ($_[0]->{'domain'}->[1]) {
-		print RESOLV "search ",join(" ", @{$_[0]->{'domain'}}),"\n";
-		}
-	else {
-		print RESOLV "domain $_[0]->{'domain'}->[0]\n";
-		}
-	}
-foreach (@resolv) {
-	print RESOLV $_ if (!/^\s*(nameserver|domain|search)\s+/);
-	}
-close(RESOLV);
-&unlock_file("/etc/resolv.conf");
-
-&lock_file("/etc/nsswitch.conf");
-open(SWITCH, "/etc/nsswitch.conf");
-local @switch = <SWITCH>;
-close(SWITCH);
-open(SWITCH, ">/etc/nsswitch.conf");
-foreach (@switch) {
-	if (/^\s*hosts:\s+/) {
-		print SWITCH "hosts:\t$_[0]->{'order'}\n";
-		}
-	else { print SWITCH $_; }
-	}
-close(SWITCH);
-&unlock_file("/etc/nsswitch.conf");
-}
-
-$max_dns_servers = 3;
-
-# order_input(&dns)
-# Returns HTML for selecting the name resolution order
-sub order_input
-{
-if ($_[0]->{'order'} =~ /\[/) {
-	# Using a complex resolve list
-	return "<input name=order size=45 value=\"$_[0]->{'order'}\">\n";
-	}
-else {
-	# Can select by menus
-	local @o = split(/\s+/, $_[0]->{'order'});
-	@o = map { s/nis\+/nisplus/; s/yp/nis/; $_; } @o;
-	local ($rv, $i, $j);
-	local @srcs = ( "", "files", "dns", "nis", "nisplus", "ldap", "db" );
-	local @srcn = ( "", "Hosts", "DNS", "NIS", "NIS+", "LDAP", "DB" );
-	for($i=1; $i<@srcs; $i++) {
-		local $ii = $i-1;
-		$rv .= "<select name=order_$ii>\n";
-		for($j=0; $j<@srcs; $j++) {
-			$rv .= sprintf "<option value=\"%s\" %s>%s\n",
-					$srcs[$j],
-					$o[$ii] eq $srcs[$j] ? "selected" : "",
-					$srcn[$j] ? $srcn[$j] : "&nbsp;";
-			}
-		$rv .= "</select>\n";
-		}
-	return $rv;
-	}
-}
-
-# parse_order(&dns)
-# Parses the form created by order_input()
-sub parse_order
-{
-if (defined($in{'order'})) {
-	$in{'order'} =~ /\S/ || &error($text{'dns_eorder'});
-	$_[0]->{'order'} = $in{'order'};
-	}
-else {
-	local($i, @order);
-	for($i=0; defined($in{"order_$i"}); $i++) {
-		push(@order, $in{"order_$i"}) if ($in{"order_$i"});
-		}
-	$_[0]->{'order'} = join(" ", @order);
-	}
-}
-
 # get_hostname()
 sub get_hostname
 {
@@ -383,11 +261,9 @@ return &get_system_hostname(1);
 sub save_hostname
 {
 &system_logged("hostname $_[0] >/dev/null 2>&1");
-&lock_file("/etc/HOSTNAME");
-open(HOST, ">/etc/HOSTNAME");
-print HOST $_[0],"\n";
-close(HOST);
-&unlock_file("/etc/HOSTNAME");
+&open_lock_tempfile(HOST, ">/etc/HOSTNAME");
+&print_tempfile(HOST, $_[0],"\n");
+&close_tempfile(HOST);
 }
 
 sub routing_config_files
@@ -397,7 +273,7 @@ return ( $inet_conf );
 
 sub routing_input
 {
-open(INIT, $inet_conf);
+&open_readfile(INIT, $inet_conf);
 while(<INIT>) {
 	s/\r|\n//g;
         s/#.*$//;
