@@ -8,7 +8,7 @@ $min_virtual_number = 1;
 sub active_interfaces
 {
 local(@rv, @lines, $l);
-&open_execute_command(IFC, "ifconfig -a", 1);
+&open_execute_command(IFC, "ifconfig -a", 1, 1);
 while(<IFC>) {
 	s/\r|\n//g;
 	if (/^\S+:/) { push(@lines, $_); }
@@ -154,9 +154,7 @@ sub delete_interface
 {
 local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
 				       : $_[0]->{'name'};
-&lock_file("/etc/hostname.$name");
-unlink(&translate_filename("/etc/hostname.$name"));
-&unlock_file("/etc/hostname.$name");
+&unlink_logged("/etc/hostname.$name");
 }
 
 # iface_type(name)
@@ -344,15 +342,17 @@ return $d;
 # save_domainname(domain)
 sub save_domainname
 {
-&execute_command("domainname ".quotemeta($_[0]));
+&system_logged("domainname ".quotemeta($_[0]));
+&lock_file("/etc/defaultdomain");
 if ($_[0]) {
 	&open_tempfile(DOMAIN, ">/etc/defaultdomain");
 	&print_tempfile(DOMAIN, $_[0],"\n");
 	&close_tempfile(DOMAIN);
 	}
 else {
-	unlink(&translate_filename("/etc/defaultdomain"));
+	&unlink_file("/etc/defaultdomain");
 	}
+&unlock_file("/etc/defaultdomain");
 }
 
 sub routing_config_files
@@ -408,7 +408,7 @@ if (@defrt) {
 	&close_tempfile(DEFRT);
 	}
 else {
-	unlink(&translate_filename("/etc/defaultrouter"));
+	&unlink_file("/etc/defaultrouter");
 	}
 &unlock_file("/etc/defaultrouter");
 
@@ -416,15 +416,15 @@ else {
 &lock_file("/etc/notrouter");
 if ($in{'router'} == 0) {
 	&create_empty_file("/etc/gateways");
-	unlink(&translate_filename("/etc/notrouter"));
+	&unlink_file("/etc/notrouter");
 	}
 elsif ($in{'router'} == 2) {
 	&create_empty_file("/etc/notrouter");
-	unlink(&translate_filename("/etc/gateways"));
+	&unlink_file("/etc/gateways");
 	}
 else {
-	unlink(&translate_filename("/etc/gateways"));
-	unlink(&translate_filename("/etc/notrouter"));
+	&unlink_file("/etc/gateways");
+	&unlink_file("/etc/notrouter");
 	}
 &unlock_file("/etc/gateways");
 &unlock_file("/etc/notrouter");
@@ -469,7 +469,7 @@ if ($_[0]) {
 	&close_tempfile(DEF);
 	}
 else {
-	unlink(&translate_filename("/etc/defaultrouter"));
+	&unlink_file("/etc/defaultrouter");
 	}
 &unlock_file("/etc/defaultrouter");
 }
@@ -479,7 +479,7 @@ else {
 sub list_routes
 {
 local @rv;
-&open_execute_command(ROUTES, "netstat -rn", 1);
+&open_execute_command(ROUTES, "netstat -rn", 1, 1);
 while(<ROUTES>) {
 	s/\s+$//;
 	if (/^([0-9\.]+|default)\s+([0-9\.]+)\s+\S+\s+\S+\s+\S+(\s+(\S+))?$/) {
@@ -515,10 +515,10 @@ if (join(" ", @infile) ne join(" ", @inmem)) {
 	# Fix up default routes
 	local $r;
 	foreach $r (@inmem) {
-		system("route delete default $r >/dev/null 2>&1");
+		&system_logged("route delete default $r >/dev/null 2>&1");
 		}
 	foreach $r (@infile) {
-		system("route add default $r >/dev/null 2>&1");
+		&system_logged("route add default $r >/dev/null 2>&1");
 		}
 	}
 }
