@@ -53,7 +53,7 @@ do 'linux-lib.pl';
 sub boot_interfaces
 {
 local(@rv, $f);
-opendir(CONF, $net_scripts_dir);
+opendir(CONF, &translate_filename($net_scripts_dir));
 while($f = readdir(CONF)) {
 	local (%conf, $b);
 	if ($f =~ /^ifcfg-([a-z0-9:\.]+)\-range([a-z0-9\.\_]+)$/) {
@@ -68,6 +68,7 @@ while($f = readdir(CONF)) {
 		$b->{'up'} = 1;
 		$b->{'edit'} = 1;
 		$b->{'index'} = scalar(@rv);
+		$b->{'file'} = "$net_scripts_dir/$f";
 		push(@rv, $b);
 		}
 	elsif ($f =~ /^ifcfg-[a-z0-9:\.]+$/ && $f !~ /\.(bak|old)$/i) {
@@ -91,6 +92,7 @@ while($f = readdir(CONF)) {
 		$b->{'bootp'} = ($conf{'BOOTPROTO'} eq 'bootp');
 		$b->{'edit'} = ($b->{'name'} !~ /^ppp|irlan/);
 		$b->{'index'} = scalar(@rv);
+		$b->{'file'} = "$net_scripts_dir/$f";
 		push(@rv, $b);
 		}
 	}
@@ -145,6 +147,10 @@ else {
 			     $_[0]->{'dhcp'} ? "dhcp" : "none";
 	}
 &write_env_file("$net_scripts_dir/ifcfg-$name", \%conf);
+if (-d &translate_filename($devices_dir)) {
+	&link_file("$net_scripts_dir/ifcfg-$name",
+		   "$devices_dir/ifcfg-$name");
+	}
 &unlock_file("$net_scripts_dir/ifcfg-$name");
 }
 
@@ -156,7 +162,10 @@ local $name = $_[0]->{'range'} ne "" ? $_[0]->{'name'}."-range".$_[0]->{'range'}
 	      $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
 				       : $_[0]->{'name'};
 &lock_file("$net_scripts_dir/ifcfg-$name");
-unlink("$net_scripts_dir/ifcfg-$name");
+&unlink_file("$net_scripts_dir/ifcfg-$name");
+if (-d &translate_filename($devices_dir)) {
+	&unlink_file("$devices_dir/ifcfg-$name");
+	}
 &unlock_file("$net_scripts_dir/ifcfg-$name");
 }
 
@@ -179,127 +188,6 @@ sub valid_boot_address
 return &check_ipaddress($_[0]);
 }
 
-# get_dns_config()
-# Returns a hashtable containing keys nameserver, domain, search & order
-sub get_dns_config
-{
-local $dns;
-open(RESOLV, "/etc/resolv.conf");
-while(<RESOLV>) {
-	s/\r|\n//g;
-	s/#.*$//;
-	if (/nameserver\s+(.*)/) {
-		push(@{$dns->{'nameserver'}}, split(/\s+/, $1));
-		}
-	elsif (/domain\s+(\S+)/) {
-		$dns->{'domain'} = [ $1 ];
-		}
-	elsif (/search\s+(.*)/) {
-		$dns->{'domain'} = [ split(/\s+/, $1) ];
-		}
-	}
-close(RESOLV);
-open(SWITCH, "/etc/nsswitch.conf");
-while(<SWITCH>) {
-	s/\r|\n//g;
-	if (/^\s*hosts:\s+(.*)/) {
-		$dns->{'order'} = $1;
-		}
-	}
-close(SWITCH);
-return $dns;
-}
-
-# save_dns_config(&config)
-# Writes out the resolv.conf and nsswitch.conf files
-sub save_dns_config
-{
-&lock_file("/etc/resolv.conf");
-open(RESOLV, "/etc/resolv.conf");
-local @resolv = <RESOLV>;
-close(RESOLV);
-open(RESOLV, ">/etc/resolv.conf");
-foreach (@{$_[0]->{'nameserver'}}) {
-	print RESOLV "nameserver $_\n";
-	}
-if ($_[0]->{'domain'}) {
-	if ($_[0]->{'domain'}->[1]) {
-		print RESOLV "search ",join(" ", @{$_[0]->{'domain'}}),"\n";
-		}
-	else {
-		print RESOLV "domain $_[0]->{'domain'}->[0]\n";
-		}
-	}
-foreach (@resolv) {
-	print RESOLV $_ if (!/^\s*(nameserver|domain|search)\s+/);
-	}
-close(RESOLV);
-&unlock_file("/etc/resolv.conf");
-
-&lock_file("/etc/nsswitch.conf");
-open(SWITCH, "/etc/nsswitch.conf");
-local @switch = <SWITCH>;
-close(SWITCH);
-open(SWITCH, ">/etc/nsswitch.conf");
-foreach (@switch) {
-	if (/^\s*hosts:\s+/) {
-		print SWITCH "hosts:\t$_[0]->{'order'}\n";
-		}
-	else { print SWITCH $_; }
-	}
-close(SWITCH);
-&unlock_file("/etc/nsswitch.conf");
-}
-
-$max_dns_servers = 3;
-
-# order_input(&dns)
-# Returns HTML for selecting the name resolution order
-sub order_input
-{
-if ($_[0]->{'order'} =~ /\[/) {
-	# Using a complex resolve list
-	return "<input name=order size=45 value=\"$_[0]->{'order'}\">\n";
-	}
-else {
-	# Can select by menus
-	local @o = split(/\s+/, $_[0]->{'order'});
-	@o = map { s/nis\+/nisplus/; s/yp/nis/; $_; } @o;
-	local ($rv, $i, $j);
-	local @srcs = ( "", "files", "dns", "nis", "nisplus", "ldap", "db" );
-	local @srcn = ( "", "Hosts", "DNS", "NIS", "NIS+", "LDAP", "DB" );
-	for($i=1; $i<@srcs; $i++) {
-		local $ii = $i-1;
-		$rv .= "<select name=order_$ii>\n";
-		for($j=0; $j<@srcs; $j++) {
-			$rv .= sprintf "<option value=\"%s\" %s>%s\n",
-					$srcs[$j],
-					$o[$ii] eq $srcs[$j] ? "selected" : "",
-					$srcn[$j] ? $srcn[$j] : "&nbsp;";
-			}
-		$rv .= "</select>\n";
-		}
-	return $rv;
-	}
-}
-
-# parse_order(&dns)
-# Parses the form created by order_input()
-sub parse_order
-{
-if (defined($in{'order'})) {
-	$in{'order'} =~ /\S/ || &error($text{'dns_eorder'});
-	$_[0]->{'order'} = $in{'order'};
-	}
-else {
-	local($i, @order);
-	for($i=0; defined($in{"order_$i"}); $i++) {
-		push(@order, $in{"order_$i"}) if ($in{"order_$i"});
-		}
-	$_[0]->{'order'} = join(" ", @order);
-	}
-}
-
 # get_hostname()
 sub get_hostname
 {
@@ -309,33 +197,45 @@ return &get_system_hostname(1);
 # save_hostname(name)
 sub save_hostname
 {
+local $old = &get_hostname();
 local %conf;
 &system_logged("hostname $_[0] >/dev/null 2>&1");
-&lock_file("/etc/HOSTNAME");
-open(HOST, ">/etc/HOSTNAME");
-print HOST $_[0],"\n";
-close(HOST);
-&unlock_file("/etc/HOSTNAME");
+&open_lock_tempfile(HOST, ">/etc/HOSTNAME");
+&print_tempfile(HOST, $_[0],"\n");
+&close_tempfile(HOST);
 &lock_file($network_config);
 &read_env_file($network_config, \%conf);
 $conf{'HOSTNAME'} = $_[0];
 &write_env_file($network_config, \%conf);
 &unlock_file($network_config);
+
+# If any ifcfg-XXX files have the old hostname in DHCP_HOSTNAME, fix it
+foreach my $b (&boot_interfaces()) {
+	local %ifc;
+	&read_env_file($b->{'file'}, \%ifc);
+	if ($ifc{'DHCP_HOSTNAME'} eq $old) {
+		$ifc{'DHCP_HOSTNAME'} = $_[0];
+		&lock_file($b->{'file'});
+		&write_env_file($b->{'file'}, \%ifc);
+		&unlock_file($b->{'file'});
+		}
+	}
 }
 
 # get_domainname()
 sub get_domainname
 {
-local $d = `domainname`;
+local $d;
+&execute_command("domainname", undef, \$d, undef);
 chop($d);
-return $d eq "(none)" ? "" : $d;
+return $d;
 }
 
 # save_domainname(domain)
 sub save_domainname
 {
 local %conf;
-system("domainname \"$_[0]\" >/dev/null 2>&1");
+&execute_command("domainname ".quotemeta($_[0]));
 &read_env_file($network_config, \%conf);
 if ($_[0]) {
 	$conf{'NISDOMAIN'} = $_[0];
@@ -346,6 +246,26 @@ else {
 &write_env_file($network_config, \%conf);
 }
 
+sub routing_config_files
+{
+local @rv = ( $network_config, $sysctl_config );
+if (!$supports_dev_routes) {
+	push(@rv, $static_route_config);
+	}
+else {
+	local $f;
+	opendir(DIR, &translate_filename($devices_dir));
+	while($f = readdir(DIR)) {
+		if ($f =~ /^([a-z]+\d*(\.\d+)?(:\d+)?)\.route$/ ||
+		    $f =~ /^route\-([a-z]+\d*(\.\d+)?(:\d+)?)$/) {
+			push(@rv, "$devices_dir/$f");
+			}
+		}
+	closedir(DIR);
+	}
+return @rv;
+}
+
 sub routing_input
 {
 local (%conf, @st, @hr, %sysctl);
@@ -420,7 +340,7 @@ else {
 
 if (!$supports_dev_routes) {
 	# get static routes from single file
-	open(STATIC, $static_route_config);
+	&open_readfile(STATIC, $static_route_config);
 	while(<STATIC>) {
 		if (/(\S+)\s+net\s+(\S+)\s+netmask\s+(\S+)\s+gw\s+(\S+)/) {
 			push(@st, [ $1, $2, $3, $4 ]);
@@ -440,7 +360,7 @@ if (!$supports_dev_routes) {
 else {
 	# get static routes from per-interface files
 	local $f;
-	opendir(DIR, $devices_dir);
+	opendir(DIR, &translate_filename($devices_dir));
 	while($f = readdir(DIR)) {
 		if ($f =~ /^([a-z]+\d*(\.\d+)?(:\d+)?)\.route$/ ||
 		    $f =~ /^route\-([a-z]+\d*(\.\d+)?(:\d+)?)$/) {
@@ -557,6 +477,7 @@ else {
 	$sysctl{'net.ipv4.ip_forward'} = $in{'forward'};
 	}
 
+# Parse static and local routes
 for($i=0; defined($dev = $in{"dev_$i"}); $i++) {
 	next if (!$dev);
 	$net = $in{"net_$i"}; $netmask = $in{"netmask_$i"}; $gw = $in{"gw_$i"};
@@ -589,25 +510,19 @@ for($i=0; defined($dev = $in{"ldev_$i"}); $i++) {
 	}
 if (!$supports_dev_routes) {
 	# Write to a single file
-	&lock_file($static_route_config);
-	open(STATIC, ">$static_route_config");
-	print STATIC @st;
-	close(STATIC);
-	&unlock_file($static_route_config);
+	&open_lock_tempfile(STATIC, ">$static_route_config");
+	&print_tempfile(STATIC, @st);
+	&close_tempfile(STATIC);
 	}
 else {
 	# Write to one file per interface (delete old, then save new/updated)
 	local $f;
-	opendir(DIR, $devices_dir);
+	opendir(DIR, &translate_filename($devices_dir));
 	while($f = readdir(DIR)) {
 		if (($f =~ /^([a-z]+\d*(\.\d+)?(:\d+)?)\.route$/ ||
 		     $f =~ /^route\-([a-z]+\d*(\.\d+)?(:\d+)?)$/) && !$st{$1}) {
-			&lock_file("$devices_dir/$f");
-			unlink("$devices_dir/$f");
-			&unlock_file("$devices_dir/$f");
-			&lock_file("$net_scripts_dir/$f");
-			unlink("$net_scripts_dir/$f");
-			&unlock_file("$net_scripts_dir/$f");
+			&unlink_logged("$devices_dir/$f");
+			&unlink_logged("$net_scripts_dir/$f");
 			}
 		}
 	closedir(DIR);
@@ -623,7 +538,7 @@ else {
 		&write_env_file("$devices_dir/$f", \%rfile);
 		&unlock_file("$devices_dir/$f");
 		&lock_file("$net_scripts_dir/$f");
-		link("$devices_dir/$f", "$net_scripts_dir/$f");
+		&link_file("$devices_dir/$f", "$net_scripts_dir/$f");
 		&unlock_file("$net_scripts_dir/$f");
 		}
 	}
@@ -831,5 +746,27 @@ $in{'num'} =~ /^\d+$/ || &error($text{'range_enum'});
 $_[0]->{'num'} = $in{'num'};
 }
 
+# get_dhcp_hostname()
+# Returns 0 if the hostname is not set by DHCP, 1 if it is, or -1 if this
+# feature is not supported on this OS.
+sub get_dhcp_hostname
+{
+return -1 if ($gconfig{'os_type'} ne 'redhat-linux' ||
+	      $gconfig{'os_version'} < 11);
+local @boot = &boot_interfaces();
+local ($eth) = grep { $_->{'fullname'} =~ /^eth\d+$/ } @boot;
+return -1 if (!$eth);
+local %eth;
+&read_env_file($eth->{'file'}, \%eth);
+return $eth{'DHCP_HOSTNAME'} ne &get_system_hostname();
+}
+
+# save_dhcp_hostname(set)
+# If called with a parameter of 0, the hostname is fixed and not set by
+# DHCP. If called with 1, the hostname is chosen by DHCP.
+sub save_dhcp_hostname
+{
+}
+
 1;
 
