@@ -146,8 +146,8 @@ EOF
 			exit 10;
 			}
 		print "\n";
-		$ver = [ $names_to_real{$name}, $vnum,
-			 $name, $vnum ];
+		$ver = [ $name, $vnum,
+			  $names_to_real{$name}, $vnum ];
 		}
 	}
 
