@@ -43,9 +43,9 @@ for($i=0; defined($in{"services_$i"}); $i++) {
 			       $in{"users_$i"}, $in{"times_$i"}),"\n");
 	}
 &lock_file($file);
-open(FILE, ">$file");
-print FILE @lines;
-close(FILE);
+&open_tempfile(FILE, ">$file");
+&print_tempfile(FILE, @lines);
+&close_tempfile(FILE);
 &unlock_file($file);
 }
 
