@@ -22,10 +22,8 @@ if (&foreign_installed("useradmin") != 1) {
 	&errordie("Users and Groups module is not supported on this OS");
 	}
 
-# Find the user
-&foreign_require("useradmin", "user-lib.pl");
-@ulist = &useradmin::list_users();
-($user) = grep { $_->{'user'} eq $ARGV[0] } @ulist;
+# Find the user, either in local password file or LDAP
+$user = &find_user($ARGV[0]);
 $user || &errordie("User $ARGV[0] does not exist");
 
 $| = 1;
@@ -52,29 +50,7 @@ $err = &useradmin::check_password_restrictions($pass, $ARGV[0]);
 &errordie($err) if ($err);
 
 # Do the change!
-$user->{'olduser'} = $user->{'user'};
-$user->{'pass'} = &useradmin::encrypt_password($pass);
-$user->{'passmode'} = 3;
-$user->{'plainpass'} = $pass;
-$pft = &useradmin::passfiles_type();
-if ($pft == 2 || $pft == 5) {
-	if ($in{'expire'}) {
-		$user->{'change'} = 0;
-		}
-	else {
-		$user->{'change'} = int(time() / (60*60*24));
-		}
-	}
-elsif ($pft == 4) {
-	$user->{'change'} = time();
-	}
-&useradmin::lock_user_files();
-&useradmin::set_user_envs($user, 'MODIFY_USER', $pass);
-&useradmin::making_changes();
-&useradmin::modify_user($user, $user);
-&useradmin::made_changes();
-&useradmin::unlock_user_files();
-&useradmin::other_modules("useradmin_modify_user", $user);
+&change_password($user, $pass, 1);
 
 # All done
 exit(0);
