@@ -9,7 +9,12 @@ local($pcmd, $line, $i, %pidmap, @plist, $dummy, @w, $_);
 local $out = `ps V`;
 if ($out =~ /version\s+(\d+)\./ && $1 >= 2 || $out =~ /version\s+\./) {
 	# New version of ps, as found in redhat 6
-	open(PS, "ps --cols 500 -eo user,ruser,group,rgroup,pid,ppid,pgid,pcpu,vsz,nice,etime,time,stime,tty,args 2>/dev/null |");
+	local $width;
+	if ($1 >= 3.2) {
+		# Use width format character if allowed
+		$width = ":100";
+		}
+	open(PS, "ps --cols 500 -eo user$width,ruser$width,group$width,rgroup$width,pid,ppid,pgid,pcpu,vsz,nice,etime,time,stime,tty,args 2>/dev/null |");
 	$dummy = <PS>;
 	for($i=0; $line=<PS>; $i++) {
 		chop($line);
@@ -89,6 +94,7 @@ return @plist;
 # renice_proc(pid, nice)
 sub renice_proc
 {
+return undef if (&is_readonly_mode());
 $out = `renice $_[1] -p $_[0] 2>&1`;
 if ($?) { return $out; }
 return undef;
@@ -155,7 +161,6 @@ else {
 		$devstyle = 0;
 		}
 	local ($pty, $tty);
-	print STDERR "considering ",scalar(@ptys)," ptys\n";
 	foreach $pty (@ptys) {
 		open(PTY, "+>$pty") || next;
 		local $tty = $pty;
@@ -174,13 +179,11 @@ else {
 			}
 		else {
 			# Must open now ..
-			print STDERR "opening tty $pty $tty\n";
 			open(TTY, "+>$tty");
 			select(TTY); $| = 1; select($old);
 			return (*PTY, *TTY, $pty, $tty);
 			}
 		}
-	print STDERR "no tty found!!\n";
 	return ();
 	}
 }
