@@ -3,15 +3,17 @@
 
 do '../web-lib.pl';
 &init_config();
-require '../ui-lib.pl';
+do '../ui-lib.pl';
 if ($module_info{'usermin'} && !$ENV{'FOREIGN_MODULE_NAME'}) {
 	&switch_to_remote_user();
 	}
 do "$config{ps_style}-lib.pl";
 use POSIX;
+use Config;
 if ($module_info{'usermin'}) {
 	%access = ( 'edit' => 1,
-		    'run' => 1 );
+		    'run' => 1,
+		    'users' => 'x' );
 	$no_module_config = 1;
 	$user_processes_only = 1;
 	$index_file = "$user_module_config_directory/index";
@@ -21,6 +23,31 @@ else {
 	map { $hide{$_}++ } split(/\s+/, $access{'hide'});
 	$index_file = "$module_config_directory/index";
 	$user_processes_only = $access{'only'};
+	if (!defined($access{'users'})) {
+		$access{'users'} = $access{'uid'} < 0 ? "x" :
+				   $access{'uid'} == 0 ? "*" :
+					getpwuid($access{'uid'});
+		}
+	}
+if ($access{'run'}) {
+	if ($access{'users'} eq "*") {
+		$default_run_user = "root";
+		}
+	elsif (&can_edit_process($remote_user)) {
+		$default_run_user = $remote_user;
+		}
+	else {
+		local @canu = split(/\s+/, $access{'users'});
+		if ($canu[0] =~ /^\@(.*)$/) {
+			$default_run_user = undef;
+			}
+		elsif ($can[0] =~ /^(\d+)\-(\d+)$/) {
+			$default_run_user = getpwuid($1);
+			}
+		else {
+			$default_run_user = $canu[0];
+			}
+		}
 	}
 
 sub process_info
@@ -86,98 +113,124 @@ elsif ($access{'uid'}) {
 # process stops running. Returns the number of bytes read.
 sub safe_process_exec
 {
-# setup pipes and fork the process
-local $chld = $SIG{'CHLD'};
-$SIG{'CHLD'} = \&safe_exec_reaper;
-pipe(OUTr, OUTw);
-pipe(INr, INw);
-local $pid = fork();
-if (!$pid) {
-	#setsid();
-	untie(*STDIN);
-	untie(*STDOUT);
-	untie(*STDERR);
-	open(STDIN, "<&INr");
-	open(STDOUT, ">&OUTw");
-	open(STDERR, ">&OUTw");
-	$| = 1;
-	close(OUTr); close(INw);
+if (&is_readonly_mode()) {
+	# Veto command in readonly mode
+	return 0;
+	}
 
-	if ($_[1]) {
-		if (defined($_[2])) {
-			# switch to given UID and GID
-			$( = $_[2]; $) = "$_[2] $_[2]";
-			($>, $<) = ($_[1], $_[1]);
+if ($gconfig{'os_type'} eq 'windows') {
+	# For Windows, just run the command and read output
+	local $temp = &transname();
+	open(TEMP, ">$temp");
+	print TEMP $_[4];
+	close(TEMP);
+	&open_execute_command(OUT, "$_[0] <$temp 2>&1", 1);
+	local $fh = $_[3];
+	while(<OUT>) {
+		if ($_[5]) {
+			print $fh &html_escape($_);
 			}
 		else {
-			# switch to UID and all GIDs
-			local @u = getpwuid($_[1]);
-			$( = $u[3];
-			$) = "$u[3] ".join(" ", $u[3], &other_groups($u[0]));
-			($>, $<) = ($u[2], $u[2]);
+			print $fh $_;
 			}
 		}
-
-	# run the command
-	delete($ENV{'FOREIGN_MODULE_NAME'});
-	delete($ENV{'SCRIPT_NAME'});
-	exec("/bin/sh", "-c", $_[0]);
-	print "Exec failed : $!\n";
-	exit 1;
+	close(OUT);
+	return $got;
 	}
-close(OUTw); close(INr);
+else {
+	# setup pipes and fork the process
+	local $chld = $SIG{'CHLD'};
+	$SIG{'CHLD'} = \&safe_exec_reaper;
+	pipe(OUTr, OUTw);
+	pipe(INr, INw);
+	local $pid = fork();
+	if (!$pid) {
+		#setsid();
+		untie(*STDIN);
+		untie(*STDOUT);
+		untie(*STDERR);
+		open(STDIN, "<&INr");
+		open(STDOUT, ">&OUTw");
+		open(STDERR, ">&OUTw");
+		$| = 1;
+		close(OUTr); close(INw);
 
-# Feed input (if any)
-print INw $_[4];
-close(INw);
-
-# Read and show output
-local $fn = fileno(OUTr);
-local $got = 0;
-local $out = $_[3];
-local $line;
-local $start = time();
-$safe_process_exec_timeout = 0;
-while(1) {
-	local ($rmask, $buf);
-	vec($rmask, $fn, 1) = 1;
-	local $sel = select($rmask, undef, undef, 1);
-	if ($sel > 0 && vec($rmask, $fn, 1)) {
-		# got something to read.. print it
-		sysread(OUTr, $buf, 1024) || last;
-		$got += length($buf);
-		if ($_[5]) {
-			$buf = &html_escape($buf);
-			}
-		if ($_[6]) {
-			# Convert backspaces and returns and escapes
-			$line .= $buf;
-			while($line =~ s/^([^\n]*\n)//) {
-				local $one = $1;
-				while($one =~ s/.\010//) { }
-				$one =~ s/\033[^m]+m//g;
-				print $out $one;
+		if ($_[1]) {
+			if (defined($_[2])) {
+				# switch to given UID and GID
+				$( = $_[2]; $) = "$_[2] $_[2]";
+				($>, $<) = ($_[1], $_[1]);
+				}
+			else {
+				# switch to UID and all GIDs
+				local @u = getpwuid($_[1]);
+				$( = $u[3];
+				$) = "$u[3] ".join(" ", $u[3], &other_groups($u[0]));
+				($>, $<) = ($u[2], $u[2]);
 				}
 			}
-		else {
-			print $out $buf;
+
+		# run the command
+		delete($ENV{'FOREIGN_MODULE_NAME'});
+		delete($ENV{'SCRIPT_NAME'});
+		exec("/bin/sh", "-c", $_[0]);
+		print "Exec failed : $!\n";
+		exit 1;
+		}
+	close(OUTw); close(INr);
+
+	# Feed input (if any)
+	print INw $_[4];
+	close(INw);
+
+	# Read and show output
+	local $fn = fileno(OUTr);
+	local $got = 0;
+	local $out = $_[3];
+	local $line;
+	local $start = time();
+	$safe_process_exec_timeout = 0;
+	while(1) {
+		local ($rmask, $buf);
+		vec($rmask, $fn, 1) = 1;
+		local $sel = select($rmask, undef, undef, 1);
+		if ($sel > 0 && vec($rmask, $fn, 1)) {
+			# got something to read.. print it
+			sysread(OUTr, $buf, 1024) || last;
+			$got += length($buf);
+			if ($_[5]) {
+				$buf = &html_escape($buf);
+				}
+			if ($_[6]) {
+				# Convert backspaces and returns and escapes
+				$line .= $buf;
+				while($line =~ s/^([^\n]*\n)//) {
+					local $one = $1;
+					while($one =~ s/.\010//) { }
+					$one =~ s/\033[^m]+m//g;
+					print $out $one;
+					}
+				}
+			else {
+				print $out $buf;
+				}
+			}
+		elsif ($sel == 0) {
+			# nothing to read. maybe the process is done, and a subprocess
+			# is hanging things up
+			last if (!kill(0, $pid));
+			}
+		if ($_[7] && time() - $start > $_[7]) {
+			# Timeout exceeded - kill the process
+			kill(KILL, $pid);
+			$safe_process_exec_timeout = 1;
 			}
 		}
-	elsif ($sel == 0) {
-		# nothing to read. maybe the process is done, and a subprocess
-		# is hanging things up
-		last if (!kill(0, $pid));
-		}
-	if ($_[7] && time() - $start > $_[7]) {
-		# Timeout exceeded - kill the process
-		kill(KILL, $pid);
-		$safe_process_exec_timeout = 1;
-		}
+	close(OUTr);
+	print $out $line;
+	$SIG{'CHLD'} = $chld;
+	return $got;
 	}
-close(OUTr);
-print $out $line;
-$SIG{'CHLD'} = $chld;
-return $got;
 }
 
 # safe_process_exec_logged(..)
@@ -201,6 +254,12 @@ do {    local $oldexit = $?;
 # Starts the given command in a new pty and returns the pty filehandle and PID
 sub pty_process_exec
 {
+local ($cmd, $uid, $gid) = @_;
+if (&is_readonly_mode()) {
+	# When in readonly mode, don't run the command
+	$cmd = "/bin/true";
+	}
+
 eval "use IO::Pty";
 if (!$@) {
 	# Use the IO::Pty perl module if installed
@@ -229,7 +288,7 @@ if (!$@) {
 		open(STDOUT, ">$tty");
 		open(STDERR, ">&STDOUT");
 		close($ptyfh);
-		exec($_[0]);
+		exec($cmd);
 		print "Exec failed : $!\n";
 		exit 1;
 		}
@@ -273,7 +332,7 @@ else {
 		open(STDOUT, ">&$ttyfh");
 		open(STDERR, ">&STDOUT");
 		close($ptyfh);
-		exec($_[0]);
+		exec($cmd);
 		print "Exec failed : $!\n";
 		exit 1;
 		}
@@ -443,7 +502,73 @@ foreach $sp (@sp) {
 return @rv;
 }
 
+# supported_signals()
+# Returns signal names known to Perl for the kill function
+sub supported_signals
+{
+if (defined(&os_supported_signals)) {
+	return &os_supported_signals();
+	}
+else {
+	return split(/\s+/, $Config{'sig_name'});
+	}
+}
 
+# can_view_process(user)
+# Returns 1 if processes belong to this user can be seen
+sub can_view_process
+{
+local ($user) = @_;
+if ($hide{$user}) {
+	return 0;
+	}
+elsif ($user_processes_only) {
+	return &can_edit_process($user);
+	}
+else {
+	return 1;
+	}
+}
+
+# can_edit_process(user)
+# Returns 1 if processes belong to this user can be edited. The 'manage as'
+# user will still apply though.
+sub can_edit_process
+{
+local ($user) = @_;
+if (!$access{'edit'}) {
+	return 0;
+	}
+elsif ($hide{$user}) {
+	return 0;
+	}
+elsif ($access{'users'} eq '*') {
+	return 1;
+	}
+elsif ($access{'users'} eq 'x') {
+	return $user eq $remote_user;
+	}
+else {
+	local @uinfo = getpwnam($user);
+	foreach my $u (split(/\s+/, $access{'users'})) {
+		if ($u =~ /^\@(.*)$/) {
+			# Is he in this group?
+			local @ginfo = getgrnam($1);
+			return 1 if ($uinfo[3] == $ginfo[2]);
+			return 1 if (&indexof($ginfo[0],
+					      &other_groups($user)) >= 0);
+			}
+		elsif ($u =~ /^(\d+)\-(\d+)$/) {
+			# Check UID
+			return 1 if ($uinfo[2] >= $1 && $uinfo[2] <= $2);
+			}
+		else {
+			return 1 if ($u eq $user);
+			}
+		}
+	return 0;
+	}
+}
 
 1;
 
