@@ -19,6 +19,34 @@ closedir(DIR);
 return @rv;
 }
 
+# list_servers_sorted(applyacl)
+# Returns a list of servers, sorted according to the module configuration
+sub list_servers_sorted
+{
+local @servers = &list_servers();
+if ($_[0]) {
+	@servers = grep { &can_use_server($_) } @servers;
+	}
+if ($config{'sort_mode'} == 1) {
+	@servers = sort { $a->{'host'} cmp $b->{'host'} } @servers;
+	}
+elsif ($config{'sort_mode'} == 2) {
+	@servers = sort { lc($a->{'desc'} ? $a->{'desc'} : $a->{'host'}) cmp
+		   lc($b->{'desc'} ? $b->{'desc'} : $b->{'host'}) } @servers;
+	}
+elsif ($config{'sort_mode'} == 3) {
+	@servers = sort { $a->{'type'} cmp $b->{'type'} } @servers;
+	}
+elsif ($config{'sort_mode'} == 4) {
+	@servers = sort { &to_ipaddress($a->{'host'}) cmp
+			  &to_ipaddress($b->{'host'}) } @servers;
+	}
+elsif ($config{'sort_mode'} == 5) {
+	@servers = sort { $a->{'group'} cmp $b->{'group'} } @servers;
+	}
+return @servers;
+}
+
 # get_server(id)
 sub get_server
 {
