@@ -410,6 +410,8 @@ else
 			exit 14
 		fi
 	fi
+
+	# Ask the user if SSL should be used
 	if [ "$ssl" = "" ]; then
 		ssl=0
 		$perl -e 'use Net::SSLeay' >/dev/null 2>/dev/null
@@ -425,6 +427,14 @@ else
 		fi
 	fi
 
+	# Don't use SSL if missing Net::SSLeay
+	if [ "$ssl" = "1" ]; then
+		$perl -e 'use Net::SSLeay' >/dev/null 2>/dev/null
+		if [ $? != "0" ]; then
+			ssl=0
+		fi
+	fi
+
 	# Ask whether to run at boot time
 	if [ "$atboot" = "" ]; then
 		initsupp=`grep "^os_support=" "$srcdir/init/module.info" | sed -e 's/os_support=//g' | grep $os_type`
@@ -585,7 +595,7 @@ echo "kill \`cat \$pidfile\`" >>$config_dir/stop
 echo "#!/bin/sh" >>$config_dir/restart
 echo "$config_dir/stop && $config_dir/start" >>$config_dir/restart
 
-chmod 755 $config_dir/start $config_dir/stop
+chmod 755 $config_dir/start $config_dir/stop $config_dir/restart
 echo "..done"
 echo ""
 
@@ -666,7 +676,7 @@ fi
 
 if [ "$makeboot" = "1" ]; then
 	echo "Configuring Webmin to start at boot time.."
-	(cd "$wadir/init" ; WEBMIN_CONFIG=$config_dir WEBMIN_VAR=$var_dir "$wadir/init/atboot.pl")
+	(cd "$wadir/init" ; WEBMIN_CONFIG=$config_dir WEBMIN_VAR=$var_dir "$wadir/init/atboot.pl" $bootscript)
 	echo "..done"
 	echo ""
 fi
@@ -709,6 +719,7 @@ chmod 755 $config_dir/{sendmail,qmailadmin,postfix}*/autoreply.pl >/dev/null 2>&
 chmod 755 $config_dir/{sendmail,qmailadmin,postfix}*/filter.pl >/dev/null 2>&1
 chmod 755 $config_dir/squid*/squid-auth.pl >/dev/null 2>&1
 chmod 755 $config_dir/squid*/users >/dev/null 2>&1
+chmod 755 $config_dir/cron*/range.pl >/dev/null 2>&1
 chmod +r $config_dir/version
 if [ "$nochown" = "" ]; then
 	chown -R root "$wadir"
