@@ -15,7 +15,7 @@ local $tag = { 'tag' => '*',
 	       'index' => 0,
 	       'line' => 0 };
 push(@rv, $tag);
-open(CONF, $config{'syslog_conf'});
+&open_readfile(CONF, $config{'syslog_conf'});
 while($line = <CONF>) {
 	local $slnum = $lnum;
 	$line =~ s/\r|\n//g;
@@ -174,10 +174,10 @@ sub needs_m4
 {
 local $oldslash = $/;
 $/ = undef;
-open(CONF, $config{'syslog_conf'});
+&open_readfile(CONF, $config{'syslog_conf'});
 local $conf1 = <CONF>;
 close(CONF);
-open(CONF, "$config{'m4_path'} $config{'syslog_conf'} |");
+&open_execute_command(CONF, "$config{'m4_path'} $config{'syslog_conf'}", 1, 1);
 local $conf2 = <CONF>;
 close(CONF);
 $/ = $oldslash;
@@ -190,7 +190,7 @@ sub get_syslog_pid
 {
 local $pid;
 if ($config{'pid_file'}) {
-	if (open(PID, $config{'pid_file'}) &&
+	if (&open_readfile(PID, $config{'pid_file'}) &&
 	    <PID> =~ /^(\d+)$/ && kill(0, $1)) {
 		$pid = $1;
 		}
@@ -229,9 +229,29 @@ sub signal_syslog
 {
 local $pid = &get_syslog_pid();
 if ($pid) {
-	kill('HUP', $pid);
+	&kill_logged('HUP', $pid);
 	}
 }
 
+# all_log_files(file)
+# Given a filename, returns all rotated versions, ordered by oldest first
+sub all_log_files
+{
+$_[0] =~ /^(.*)\/([^\/]+)$/;
+local $dir = $1;
+local $base = $2;
+local ($f, @rv);
+opendir(DIR, &translate_filename($dir));
+foreach $f (readdir(DIR)) {
+	local $trans = &translate_filename("$dir/$f");
+	if ($f =~ /^\Q$base\E/ && -f $trans) {
+		push(@rv, "$dir/$f");
+		$mtime{"$dir/$f"} = [ stat($trans) ];
+		}
+	}
+closedir(DIR);
+return sort { $mtime{$a}->[9] <=> $mtime{$b}->[9] } @rv;
+}
+
 1;
 
