@@ -8,7 +8,7 @@ $timezones_dir = "/usr/share/zoneinfo";
 sub list_timezones
 {
 local @rv;
-open(ZONE, $timezones_file) || return ( );
+&open_readfile(ZONE, $timezones_file) || return ( );
 while(<ZONE>) {
 	s/\r|\n//g;
 	s/^\s*#.*$//;
@@ -26,7 +26,7 @@ return sort { $a->[0] cmp $b->[0] } @rv;
 # get_current_timezone()
 sub get_current_timezone
 {
-local $lnk = readlink($currentzone_link);
+local $lnk = readlink(&translate_filename($currentzone_link));
 if ($lnk) {
 	# Easy - it a link
 	$lnk =~ s/$timezones_dir\///;
@@ -42,8 +42,9 @@ else {
 sub set_current_timezone
 {
 &lock_file($currentzone_link);
-unlink($currentzone_link);
-symlink("$timezones_dir/$_[0]", $currentzone_link);
+unlink(&translate_filename($currentzone_link));
+symlink(&translate_filename("$timezones_dir/$_[0]"),
+	&translate_filename($currentzone_link));
 &unlock_file($currentzone_link);
 }
 
