@@ -36,8 +36,13 @@ printf "<input name=uedit_group size=40 value='%s'> %s<br>\n",
 	$o->{'uedit_mode'} == 5 ?
 	 join(" ", map { "".&my_getgrgid($_) } split(/\s+/, $o->{'uedit'})) :"",
 	&group_chooser_button("uedit_group", 1);
-printf "%s <input type=checkbox name=uedit_sec value=1 %s> %s</td></tr>\n",
+printf "%s <input type=checkbox name=uedit_sec value=1 %s> %s<br>\n",
 	"&nbsp;" x 5, $o->{'uedit_sec'} ? 'checked' : '',$text{'acl_uedit_sec'};
+printf "<input type=radio name=uedit_mode value=7 %s> $text{'acl_uedit_re'}\n",
+	$o->{'uedit_mode'} == 7 ? "checked" : "";
+printf "<input name=uedit_re size=40 value='%s'> %s<br>\n",
+	$o->{'uedit_mode'} == 7 ? $o->{'uedit_re'} : "";
+print "</td> </tr>\n";
 
 print "<tr> <td><b>$text{'acl_ucreate'}</b></td> <td>\n";
 printf "<input type=radio name=ucreate value=1 %s> $text{'yes'}\n",
@@ -117,6 +122,12 @@ printf "<input type=checkbox name=autohome value=1 %s> %s</td> </tr>\n",
 	$o->{'autohome'} ? "checked" : "",
 	$text{'acl_autohome'};
 
+print "<tr> <td><b>$text{'acl_udelete'}</b></td> <td>\n";
+printf "<input type=radio name=udelete value=1 %s> $text{'yes'}\n",
+	$o->{'udelete'} ? "checked" : "";
+printf "<input type=radio name=udelete value=0 %s> $text{'no'}</td> </tr>\n",
+	$o->{'udelete'} ? "" : "checked";
+
 print "<tr> <td valign=top><b>$text{'acl_delhome'}</b></td>\n";
 print "<td colspan=3>",&ui_radio("delhome", $o->{'delhome'},
 			  [ [ 2, $text{'acl_option'} ],
@@ -170,9 +181,9 @@ printf "<input type=radio name=gcreate value=2 %s> $text{'acl_gnew'}\n",
 printf "<input type=radio name=gcreate value=0 %s> $text{'no'}</td> </tr>\n",
 	$o->{'gcreate'}==0 ? "checked" : "";
 
-print "<tr> <td><b>$text{'acl_gid'}</b></td>\n";
-print "<td><input name=lowgid size=6 value='$o->{'lowgid'}'> -\n";
-print "<input name=higid size=6 value='$o->{'higid'}'></td> <td>\n";
+print "<tr> <td valign=top><b>$text{'acl_gid'}</b></td>\n";
+print "<td colspan=3><input name=lowgid size=6 value='$o->{'lowgid'}'> -\n";
+print "<input name=higid size=6 value='$o->{'higid'}'><br>\n";
 printf "<input type=checkbox name=autogid value=1 %s> %s<br>\n",
 	$o->{'autogid'} ? "checked" : "", $text{'acl_autogid'};
 printf "<input type=checkbox name=calcgid value=1 %s> %s<br>\n",
@@ -184,6 +195,12 @@ printf "<input type=checkbox name=gmultiple value=1 %s> %s<br>\n",
 printf "<input type=checkbox name=ggid value=1 %s> %s</td> </tr>\n",
 	$o->{'ggid'} ? "checked" : "", $text{'acl_ggid'};
 
+print "<tr> <td><b>$text{'acl_gdelete'}</b></td> <td>\n";
+printf "<input type=radio name=gdelete value=1 %s> $text{'yes'}\n",
+	$o->{'gdelete'} ? "checked" : "";
+printf "<input type=radio name=gdelete value=0 %s> $text{'no'}</td> </tr>\n",
+	$o->{'gdelete'} ? "" : "checked";
+
 print "<tr> <td colspan=4><hr></td> </tr>\n";
 
 print "<tr><td valign=top><b>$text{'acl_logins'}</b></td> <td colspan=3>\n";
@@ -221,6 +238,7 @@ $_[0]->{'uedit'} = $in{'uedit_mode'} == 2 ? $in{'uedit_can'} :
 			     split(/\s+/, $in{'uedit_group'})) : "";
 $_[0]->{'uedit2'} = $in{'uedit_mode'} == 4 ? $in{'uedit_uid2'} : undef;
 $_[0]->{'uedit_sec'} = $in{'uedit_mode'} == 5 ? $in{'uedit_sec'} : undef;
+$_[0]->{'uedit_re'} = $in{'uedit_mode'} == 7 ? $in{'uedit_re'} : undef;
 $_[0]->{'gedit_mode'} = $in{'gedit_mode'};
 $_[0]->{'gedit'} = $in{'gedit_mode'} == 2 ? $in{'gedit_can'} :
 		   $in{'gedit_mode'} == 3 ? $in{'gedit_cannot'} :
@@ -261,5 +279,7 @@ foreach $o ('chuid', 'chgid', 'movehome', 'mothers',
 	    'makehome', 'copy', 'cothers', 'dothers') {
 	$_[0]->{$o} = $in{$o};
 	}
+$_[0]->{'udelete'} = $in{'udelete'};
+$_[0]->{'gdelete'} = $in{'gdelete'};
 }
 
