@@ -68,7 +68,7 @@ if ($pft == 1) {
 	}
 elsif ($pft == 6) {
 	# Read netinfo dump
-	&open_readfile(PASSWD, "nidump passwd '$netinfo_domain' |");
+	&open_execute_command(PASSWD, "nidump passwd '$netinfo_domain'", 1);
 	while(<PASSWD>) {
 		s/\r|\n//g;
 		if (/\S/ && !/^[#\+\-]/) {
@@ -440,7 +440,7 @@ $lnum = 0;
 local $gft = &groupfiles_type();
 if ($gft == 5) {
 	# Get groups from netinfo
-	&open_readfile(GROUP, "nidump group '$netinfo_domain' |");
+	&open_execute_command(GROUP, "nidump group '$netinfo_domain'", 1);
 	while(<GROUP>) {
 		s/\r|\n//g;
 		if (/\S/ && !/^[#\+\-]/) {
@@ -638,16 +638,18 @@ else {
 sub recursive_change
 {
 local(@list, $f, @stbuf);
-(@stbuf = stat($_[0])) || return;
-(-l $_[0]) && return;
+local $real = &translate_filename($_[0]);
+(@stbuf = stat($real)) || return;
+(-l $real) && return;
 if (($_[1] < 0 || $_[1] == $stbuf[4]) &&
     ($_[2] < 0 || $_[2] == $stbuf[5])) {
 	# Found match..
-	chown($_[3] < 0 ? $stbuf[4] : $_[3],
-	      $_[4] < 0 ? $stbuf[5] : $_[4], $_[0]);
+	&set_ownership_permissions(
+		$_[3] < 0 ? $stbuf[4] : $_[3],
+	      	$_[4] < 0 ? $stbuf[5] : $_[4], undef, $_[0]);
 	}
-if (-d $_[0]) {
-	opendir(DIR, $_[0]);
+if (-d $real) {
+	opendir(DIR, $real);
 	@list = readdir(DIR);
 	closedir(DIR);
 	foreach $f (@list) {
@@ -684,6 +686,7 @@ return undef;
 # Call some function in the useradmin_update.pl file in other modules
 sub other_modules
 {
+return if (&is_readonly_mode());	# don't even try other modules
 local($m, %minfo);
 local $func = shift(@_);
 foreach $m (&get_all_module_infos()) {
@@ -733,6 +736,9 @@ elsif ($m == 4) {
 elsif ($m == 6) {
 	return $_[1]->{'user'} eq $remote_user;
 	}
+elsif ($m == 7) {
+	return $_[1]->{'user'} =~ /$_[0]->{'uedit_re'}/;
+	}
 return 0;
 }
 
@@ -801,24 +807,25 @@ if (-b $_[0] || -c $_[0]) {
 	local $maj = int($st[6] / 256);
 	local $min = $st[6] % 256;
 	local $typ = ($st[2] & 00170000) == 0020000 ? 'c' : 'b';
-	&system_logged("mknod \"$_[1]/$base\" $typ $maj $min");
-	chown($_[2], $_[3], "$_[1]/$base");
+	&system_logged("mknod ".quotemeta("$_[1]/$base")." $typ $maj $min");
+	&set_ownership_permissions($_[2], $_[3], undef, "$_[1]/$base");
 	$nochown++;
 	}
 elsif (-l $_[0] && !$config{'copy_symlinks'}) {
 	# A symlink .. re-create it
 	local $l = readlink($_[0]);
-	&system_logged("ln -s \"$l\" \"$_[1]/$base\" >/dev/null 2>/dev/null");
+	&system_logged("ln -s ".quotemeta($l)." ".quotemeta("$_[1]/$base")." >/dev/null 2>/dev/null");
 	$opts = "-h";
 	}
 elsif (-d $_[0]) {
 	# A directory .. copy it recursively
-	&system_logged("cp -R \"$_[0]\" \"$_[1]/$base\" >/dev/null 2>/dev/null");
+	&system_logged("cp -R ".quotemeta($_[0])." ".quotemeta("$_[1]/$base")." >/dev/null 2>/dev/null");
 	}
 else {
 	# Just a normal file .. copy it
-	&system_logged("cp \"$_[0]\" \"$_[1]/$base\" >/dev/null 2>/dev/null");
-	chown($_[2], $_[3], "$_[1]/$base");
+	local @st = stat(&translate_filename($_[0]));
+	&system_logged("cp ".quotemeta($_[0])." ".quotemeta("$_[1]/$base")." >/dev/null 2>/dev/null");
+	&set_ownership_permissions($_[2], $_[3], $st[2], "$_[1]/$base");
 	$nochown++;
 	}
 &system_logged("chown $opts -R $_[2]:$_[3] \"$_[1]/$base\" >/dev/null 2>/dev/null") if (!$nochown);
@@ -1021,7 +1028,7 @@ elsif ($config{'home_style'} == 5) {
 
 sub mkdir_if_needed
 {
--d $_[0] || mkdir($_[0], 0755);
+-d $_[0] || &make_dir($_[0], 0755);
 }
 
 sub set_netinfo
@@ -1086,10 +1093,10 @@ if ($config{'passwd_dict'} && $_[0] =~ /^[A-Za-z\'\-]+$/ &&
 	return &text('usave_epasswd_dict') if (!$unknown);
 	}
 if ($config{'passwd_prog'}) {
-	# Run external program with username and password as args
+	# Run external validation program with username and password as args
 	local $qu = quotemeta($_[1]);
 	local $qp = quotemeta($_[0]);
-	local $out = `$config{'passwd_prog'} $qu $qp 2>&1 </dev/null`;
+	local $out = &backquote_command("$config{'passwd_prog'} $qu $qp 2>&1 </dev/null");
 	if ($?) {
 		return $out;
 		}
@@ -1328,6 +1335,9 @@ elsif ($access{'uedit_mode'} == 5) {
 elsif ($access{'uedit_mode'} == 6) {
 	@ulist = grep { $_->{'user'} eq $remote_user } @ulist;
 	}
+elsif ($access{'uedit_mode'} == 7) {
+	@ulist = grep { $_->{'user'} =~ /$access{'uedit_re'}/ } @ulist;
+	}
 if ($access{'view'}) {
 	# Include non-editable users in results
 	local @rv = @{$_[1]};
@@ -1592,7 +1602,9 @@ sub berkeley_cksum {
     return sprintf("%lu",${crc});;
 }
 
-# users_table(&users)
+# users_table(&users, [form], [no-last], [no-boxes])
+# Prints a table listing full user details, with checkboxes and buttons to
+# delete or disable multiple at once.
 sub users_table
 {
 local (@ginfo, %gidgrp);
@@ -1602,11 +1614,12 @@ while(@ginfo = &my_getgrent()) {
 	}
 &my_endgrent();
 local ($anyedit) = grep { !$_->{'noedit'} } @{$_[0]};
-local $lshow = $config{'last_show'};
+$anyedit = 0 if ($_[3]);
+local $lshow = !$_[2] && $config{'last_show'};
 print "<form action=mass_delete_user.cgi method=post>\n";
 if ($anyedit) {
-	print &select_all_link("d", 0),"\n";
-	print &select_invert_link("d", 0),"<br>\n";
+	print &select_all_link("d", $_[1]),"\n";
+	print &select_invert_link("d", $_[1]),"<br>\n";
 	}
 print "<table width=100% border>\n";
 print "<tr $tb> ",
@@ -1644,22 +1657,27 @@ foreach $u (@{$_[0]}) {
 	}
 print "</table>\n";
 if ($anyedit) {
-	print &select_all_link("d", 0),"\n";
-	print &select_invert_link("d", 0),"<br>\n";
+	print &select_all_link("d", $_[1]),"\n";
+	print &select_invert_link("d", $_[1]),"<br>\n";
+	}
+if ($anyedit) {
+	print "<input type=submit name=delete value='$text{'index_mass'}'>\n";
+	print "<input type=submit name=disable value='$text{'index_mass2'}'>\n";
+	print "<input type=submit name=enable value='$text{'index_mass3'}'>\n";
+	print "</form>\n";
 	}
-print "<input type=submit value='$text{'index_mass'}'>\n" if ($anyedit);
-print "</form>\n";
 }
 
-# groups_table(&groups)
+# groups_table(&groups, [form], [no-buttons])
 sub groups_table
 {
 local ($anyedit) = grep { !$_->{'noedit'} } @{$_[0]};
+$anyedit = 0 if ($_[2]);
 print "<form action=mass_delete_group.cgi method=post>\n";
 print "<table border width=100%>\n";
 if ($anyedit) {
-	print &select_all_link("d", 0),"\n";
-	print &select_invert_link("d", 0),"<br>\n";
+	print &select_all_link("d", $_[1]),"\n";
+	print &select_invert_link("d", $_[1]),"<br>\n";
 	}
 print "<tr $tb> ",
       !$anyedit ? "" : "<td><br></td> ",
@@ -1679,11 +1697,13 @@ foreach $g (@{$_[0]}) {
 	}
 print "</table>\n";
 if ($anyedit) {
-	print &select_all_link("d", 0),"\n";
-	print &select_invert_link("d", 0),"<br>\n";
+	print &select_all_link("d", $_[1]),"\n";
+	print &select_invert_link("d", $_[1]),"<br>\n";
+	}
+if ($anyedit) {
+	print "<input type=submit value='$text{'index_gmass'}'>\n";
+	print "</form>\n";
 	}
-print "<input type=submit value='$text{'index_gmass'}'>\n" if ($anyedit);
-print "</form>\n";
 }
 
 sub ifblank
@@ -1725,12 +1745,22 @@ return @rv;
 # user_link(&user)
 sub user_link
 {
+if ($_[0]->{'pass'} =~ /^\Q$disable_string\E/) {
+	$dis = "<i>".&html_escape($_[0]->{'user'})."</i>";
+	}
+else {
+	$dis = &html_escape($_[0]->{'user'});
+	}
 if ($_[0]->{'noedit'}) {
-	return &html_escape($_[0]->{'user'});
+	return $dis;
+	}
+elsif ($_[0]->{'dn'}) {
+	return "<a href='edit_user.cgi?dn=".&urlize($_[0]->{'dn'})."'>".
+	       "$dis</a>";
 	}
 else {
 	return "<a href='edit_user.cgi?num=$_[0]->{'num'}'>".
-	       &html_escape($_[0]->{'user'})."</a>";
+	       "$dis</a>";
 	}
 }
 
@@ -1740,12 +1770,72 @@ sub group_link
 if ($_[0]->{'noedit'}) {
 	return &html_escape($_[0]->{'group'});
 	}
+elsif ($_[0]->{'dn'}) {
+	return "<a href='edit_group.cgi?dn=".&urlize($_[0]->{'dn'})."'>".
+	       &html_escape($_[0]->{'group'})."</a>";
+	}
 else {
 	return "<a href='edit_group.cgi?num=$_[0]->{'num'}'>".
 	       &html_escape($_[0]->{'group'})."</a>";
 	}
 }
 
+# sort_users(&users, mode)
+# Sorts a list of users, and returns the results
+sub sort_users
+{
+local ($users, $mode) = @_;
+local @ulist = @$users;
+if ($mode == 1) {
+	@ulist = sort { $a->{'user'} cmp $b->{'user'} } @ulist;
+	}
+elsif ($mode == 2) {
+	@ulist = sort { lc($a->{'real'}) cmp lc($b->{'real'}) } @ulist;
+	}
+elsif ($mode == 3) {
+	@ulist = sort { @wa = split(/\s+/, $a->{'real'});
+			@wb = split(/\s+/, $b->{'real'});
+			lc($wa[@wa-1]) cmp lc($wb[@wb-1]) } @ulist;
+	}
+elsif ($mode == 4) {
+	@ulist = sort { $a->{'shell'} cmp $b->{'shell'} } @ulist;
+	}
+elsif ($mode == 5) {
+	@ulist = sort { $a->{'uid'} <=> $b->{'uid'} } @ulist;
+	}
+elsif ($mode == 6) {
+	@ulist = sort { $a->{'home'} cmp $b->{'home'} } @ulist;
+	}
+return @ulist;
+}
 
+# sort_groups(&groups, mode)
+sub sort_groups
+{
+local ($groups, $mode) = @_;
+local @glist = @$groups;
+if ($mode == 5) {
+	@glist = sort { $a->{'gid'} <=> $b->{'gid'} } @glist;
+	}
+elsif ($mode == 1) {
+	@glist = sort { $a->{'group'} cmp $b->{'group'} } @glist;
+	}
+return @glist;
+}
+
+# create_home_directory(&user, [real-dir])
+# Creates and chmod's the home directory for a user, or calls error on failure
+sub create_home_directory
+{
+local ($user, $home) = @_;
+$home ||= $user->{'home'};
+&lock_file($home);
+&make_dir($home, oct($config{'homedir_perms'}), 1) ||
+	&error(&text('usave_emkdir', $!));
+&set_ownership_permissions($user->{'uid'}, $user->{'gid'},
+			   oct($config{'homedir_perms'}), $home) ||
+	&error(&text('usave_echmod', $!));
+&unlock_file($home);
+}
 
 1;
