@@ -8,10 +8,14 @@ require '../ui-lib.pl';
 @cs_codes = ( 'cs_page', 'cs_text', 'cs_table', 'cs_header', 'cs_link' );
 @cs_names = map { $text{$_} } @cs_codes;
 
+$osdn_host = "prdownloads.sourceforge.net";
+$osdn_port = 80;
+
 $update_host = "www.webmin.com";
 $update_port = 80;
 $update_page = "/updates/updates.txt";
 $update_url = "http://$update_host:$update_port$update_page";
+$redirect_url = "http://$update_host/cgi-bin/redirect.cgi";
 
 $webmin_key_email = "jcameron\@webmin.com";
 $webmin_key_fingerprint = "1719 003A CE3E 5A41 E2DE  70DF D97A 3AE9 11F6 3C51";
@@ -30,6 +34,8 @@ $default_key_size = "512";
 
 $cron_cmd = "$module_config_directory/update.pl";
 
+$os_info_address = "os\@webmin.com";
+
 sub setup_ca
 {
 local $adir = &module_root_directory("acl");
@@ -200,6 +206,7 @@ else {
 	# Check if this is a valid module (a tar file of multiple module or
 	# theme directories)
 	local (%mods, %hasfile);
+	&has_command("tar") || return $text{'install_enotar'};
 	local $tar = `tar tf "$file" 2>&1`;
 	if ($?) {
 		unlink($file) if ($need_unlink);
@@ -776,6 +783,26 @@ local %rv;
 return %rv;
 }
 
+# list_operating_systems([os-list-file])
+# Returns a list of know OS's
+sub list_operating_systems
+{
+local $file = $_[0] || "$root_directory/os_list.txt";
+local @rv;
+open(OSLIST, $file);
+while(<OSLIST>) {
+	if (/^([^\t]+)\t+([^\t]+)\t+([^\t]+)\t+([^\t]+)\t+(.*)/) {
+		push(@rv, { 'realtype' => $1,
+			    'realversion' => $2,
+			    'type' => $3,
+			    'version' => $4,
+			    'code' => $5 });
+		}
+	}
+close(OSLIST);
+return @rv;
+}
+
 # shared_root_directory()
 # Returns 1 if the Webmin root directory is shared with another system, such as
 # via NFS, or in a Solaris zone. If so, updates and module installs are not
@@ -803,4 +830,42 @@ if (&running_in_zone()) {
 return 0;
 }
 
+# submit_os_info(id)
+# Send via email a message about this system's OS and Perl version. Returns
+# undef if OK, or an error message
+sub submit_os_info
+{
+if (!&foreign_installed("mailboxes", 1)) {
+	return $text{'submit_emailboxes'};
+	}
+&foreign_require("mailboxes", "mailboxes-lib.pl");
+local $mail = { 'headers' => [ [ 'From', &mailboxes::get_from_address() ],
+			       [ 'To', $os_info_address ],
+			       [ 'Subject', 'Webmin OS Information' ] ],
+		'attach' => [ {
+		   'headers' => [ [ 'Content-type', 'text/plain' ] ],
+		   'data' => "OS: $gconfig{'real_os_type'}\n".
+			     "Version: $gconfig{'real_os_version'}\n".
+		 	     "OS code: $gconfig{'os_type'}\n".
+		 	     "Version code: $gconfig{'os_version'}\n".
+			     "Perl: $]\n".
+			     "Webmin: ".&get_webmin_version()."\n".
+			     "ID: ".&get_webmin_id()."\n" } ],
+		};
+eval { &mailboxes::send_mail($mail); };
+return $@ ? $@ : undef;
+}
+
+# get_webmin_id()
+# Returns a (hopefully) unique ID for this Webmin install
+sub get_webmin_id
+{
+if (!$config{'webminid'}) {
+	local $salt = substr(time(), -2);
+	$config{'webminid'} = crypt(&get_system_hostname(), $salt);
+	&save_module_config();
+	}
+return $config{'webminid'};
+}
+
 1;
