@@ -125,15 +125,51 @@ return $access_mods{'*'} || $access_mods{$_[0]};
 # Returns the structure for some action
 sub get_action
 {
+local %index;
+&build_log_index(\%index);
 local $act;
 open(LOG, $webmin_logfile);
-while(<LOG>) {
-	if (($act = &parse_logline($_)) && $act->{'id'} eq $_[0]) {
-		return $act;
-		}
-	}
+local @idx = split(/\s+/, $index{$_[0]});
+seek(LOG, $idx[0], 0);
+local $line = <LOG>;
+local $act = &parse_logline($line);
 close(LOG);
-return undef;
+return $act->{'id'} eq $_[0] ? $act : undef;
+}
+
+# build_log_index(&index)
+# Updates the given hash with mappings between action IDs and file positions
+sub build_log_index
+{
+local ($index) = @_;
+local $ifile = "$module_config_directory/logindex";
+dbmopen(%$index, $ifile, 0600);
+local @st = stat($webmin_logfile);
+if ($st[9] > $index->{'lastchange'}) {
+	# Log has changed .. perhaps need to rebuild
+	open(LOG, $webmin_logfile);
+	if ($index->{'lastsize'} && $st[7] >= $index->{'lastsize'}) {
+		# Gotten bigger .. just add new lines
+		seek(LOG, $index->{'lastpos'}, 0);
+		}
+	else {
+		# Smaller! Need to rebuild from start
+		%$index = ( 'lastpos' => 0 );
+		}
+	while(<LOG>) {
+		if ($act = &parse_logline($_)) {
+			$index->{$act->{'id'}} = $index->{'lastpos'}." ".
+						 $act->{'time'}." ".
+						 $act->{'user'}." ".
+						 $act->{'module'}." ".
+						 $act->{'sid'};
+			}
+		$index->{'lastpos'} += length($_);
+		}
+	close(LOG);
+	$index->{'lastsize'} = $st[7];
+	$index->{'lastchange'} = $st[9];
+	}
 }
 
 1;
